from __future__ import annotations

from tcsoa.gen.BusinessObjects import TaskDependency, ScheduleMember, TCCalendar, Schedule, BusinessObject, SchedulingFixedCost, ResourceAssignment, WorkspaceObject, BillRate, ScheduleTask, TCCalendarEvent
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class FixedCostContainer(TcBaseObj):
    """
    The information need to create/update a Fixed Cost.
    
    :var name: The name of fixed cost.
    :var estimate: The estimated cost. Valid value can be empty string
    :var actual: The actual cost. Valid value can be empty string.
    :var currency: ISO-4217 code for currency of the costs.
    :var useActual: Should accrual calculations use the "actual" cost?
    :var billCode: The billing code. Valid values are determined by the LOV  {unassigned, General, Management,
    ProjectMgmt, Sales, Training, Travel, ProductDev, SoftwareDev}
    :var subCode: The billing subcode. Valid values are determined by the LOV. Values are following groups depending
    upon billcode.eg. If billcode is General then you can mentioned subcode as either of following set (unassigned,
    Accounting, Clerical, CorpAdmin, IT, Meetings, Other)                                             Following are the
    list of billcode and corresponsing valid values for sub code.  For BillCode 'unassigned' = { unassigned } For
    BillCode 'General' ={ unassigned, Accounting, Clerical, CorpAdmin, IT, Meetings, Other },
    For Bill code Management = { unassigned, Executive, ProjMgmt, Design/Plan, Meetings, Training, Other}
    For BillCode 'ProiectMgmt' ={ unassigned, Management, Meetings, Design/Plan, Training, Teaching, Clerical, Email,
    Other }
    For BillCode 'Sales' ={ unassigned, MajorAccts, General, Admin, Training, Other }
    For BillCode 'Training' ={ unassigned, Billable, Customer1, Customer2, Customer3, Region1, Region2, Region3, Other }
    For BillCode 'Travel' ={ unassigned, Billable, Region1, Region2, Region3, Other }
    For BillCode 'ProductDev' ={ unassigned, Planning, Design, Development, ProcessMgt, Validation, Other }
    For BillCode 'SoftwareDev' ={unassigned, Concept, Defination, Development, Introduction, Training, Other }
    :var billingType: The billing type. Valid value are
    { unassigned, Billable, Billed, Standard, Unbillable}
    :var accrualType: The cost accrual type (0=start, 1=prorated, 2=end).
    :var fixedCost: A reference to the cost being updated (or null when newCost).
    """
    name: str = ''
    estimate: str = ''
    actual: str = ''
    currency: str = ''
    useActual: bool = False
    billCode: str = ''
    subCode: str = ''
    billingType: str = ''
    accrualType: int = 0
    fixedCost: SchedulingFixedCost = None


@dataclass
class MembershipData(TcBaseObj):
    """
    The information needed to create a new member in a schedule.
    
    :var schedule: The schedule for the new membership. Valid value- tag of the schedule.
    :var resource: The resource to add.  (This can be a User, Group, or Discipline).valid value- tag of the resource
    :var membershipLevel: The membership level in that schedule. Valid values
    are-{0-observer,1-participant,2-coordinator}
    :var cost: The cost value (n.nnn (15.3 max))
    :var currency: The ISO-4217 currency code.
    """
    schedule: Schedule = None
    resource: BusinessObject = None
    membershipLevel: int = 0
    cost: str = ''
    currency: str = ''


@dataclass
class NewScheduleContainer(TcBaseObj):
    """
    The container for a new schedule.
    
    :var name: The name. Valid value can not be null
    :var description: The description. Valid value can be empty string
    :var taskFixedType: Task fixed type. Valid values - FIXED_WORK = 0, FIXED_DURATION = 1, FIXED_RESOURCES=2
    :var published: Indicates whether the schedule is published. Valid values - true or false
    :var notificationsEnabled: notificationsEnabled. Indicates whether notifications should be enabled. Valid values -
    true or false
    :var percentLinked: isPercentLinked? Indicates whether percentage complete should be linked to work complete. Valid
    values - true, false
    :var isTemplate: isTemplate? Indicates whether the schedule is a template. Valid values - true, false
    :var isPublic: Indicates whether the schedule is public. Valid values - true, false"
    :var type: type. This is the object_type of the schedule being created. It could be "Schedule" or any of the custom
    types created by the customer.
    :var billCode: The billCode. Valid value can be empty string
    :var billSubCode: The bill sub code. Valid value can be empty string
    :var billType: The bill type. Valid value can be empty string
    :var id: id. Valid value can not be null
    :var billRate: The bill rate. Valid value can be NULLTAG
    :var stringValueContainer: A collection of additional attributes (Optional)
    :var typedAttributesContainer: typedAttributesContainer. A container with type attribute -(optional)
    :var revID: revID. Valid value can not be null
    :var customerName: The customer's name. Valid value can be empty string
    :var customerNumber: The customer's ID. Valid value can be empty string
    :var startDate: The start Date. Valid value can not be null
    :var finishDate: The finishDate. Valid value can not be null
    :var priority: The priority. Valid values are {0-lowest,1-low,2-MediumLow,3-Medium,4-High,5-VeryHigh,6-Highest}
    :var status: The status. Valid values are  {0-Not started,1-In Progress,2-Needs
    Attention,3-Complete,4-Abandoned,5-Late}
    """
    name: str = ''
    description: str = ''
    taskFixedType: int = 0
    published: bool = False
    notificationsEnabled: bool = False
    percentLinked: bool = False
    isTemplate: bool = False
    isPublic: bool = False
    type: str = ''
    billCode: str = ''
    billSubCode: str = ''
    billType: str = ''
    id: str = ''
    billRate: BusinessObject = None
    stringValueContainer: List[StringValContainer] = ()
    typedAttributesContainer: List[TypedAttributeContainer] = ()
    revID: str = ''
    customerName: str = ''
    customerNumber: str = ''
    startDate: datetime = None
    finishDate: datetime = None
    priority: int = 0
    status: int = 0


@dataclass
class ScheduleCopyOptionsContainer(TcBaseObj):
    """
    The input information necessary to copy a schedule.
    
    :var name: The name of the new Schedule to be created.The name cannot be 'NULL' 
    
    :var description: The description text for the new Schedule to be created.
    :var typedAttributesContainer: This parameter specifies  any additional attribute values to be set on the newly
    created Schedule,scheduleRevision.
    :var copyinfo: This parameter specifies any additional metadata for the copy of the schedule. This 'DeepCopyData'
    data structure holds the relevant information regarding applicable deep copy rules.
    :var id: The id of the new Schedule to be created. The id value cannot be NULL.
    :var revId: The 'revID' of the new ScheduleRevision that will be created along with the new schedule.
    The 'revID' value cannot be NULL.
    
    :var scheduleToCopy: The tag to the schedule to copy.
    :var resetWork: This is the flag to indicate whether or not to reset the tasks' execution data (%, status, work
    complete, etc)
    :var copyBaselines: Flag to indicate whether or not to copy baselines.
    :var loadOnResponse: Flag to indicate whether or not to load the schedule in the response.
    :var iDeepCopyCount: This parameter stores the number of deep copied objects.
    :var stringValueContainer: This parameter is  to collect additional attributes for the new schedule. The container
    represents a single attribute's value.
    """
    name: str = ''
    description: str = ''
    typedAttributesContainer: List[TypedAttributeContainer] = ()
    copyinfo: List[ScheduleDeepCopyinfo] = ()
    id: str = ''
    revId: str = ''
    scheduleToCopy: Schedule = None
    resetWork: bool = False
    copyBaselines: bool = False
    loadOnResponse: bool = False
    iDeepCopyCount: int = 0
    stringValueContainer: List[StringValContainer] = ()


@dataclass
class ScheduleCopyResponse(TcBaseObj):
    """
    The response from a schedule copy call.  It includes all the data necessary to load that schedule.
    
    :var schedule: The schedule.
    :var scheduleTask: The schedule tasks.
    :var resourceAssignment: The task assignments.
    :var taskDependency: The task dependencies.
    :var calendar: The calendars asssociated with the schedule and its members.
    :var calendarEvent: The calendar events associated with the calendars.
    """
    schedule: Schedule = None
    scheduleTask: List[ScheduleTask] = ()
    resourceAssignment: List[ResourceAssignment] = ()
    taskDependency: List[TaskDependency] = ()
    calendar: List[TCCalendar] = ()
    calendarEvent: List[TCCalendarEvent] = ()


@dataclass
class ScheduleCopyResponses(TcBaseObj):
    """
    Container to hold multiple ScheduleCopyResponse objects.
    
    :var scheduleResponse: A collection of ScheduleCopyResponses.
    :var serviceData: The ServiceData.
    """
    scheduleResponse: List[ScheduleCopyResponse] = ()
    serviceData: ServiceData = None


@dataclass
class ScheduleDeepCopyinfo(TcBaseObj):
    """
    The DeepCopyData data structure holds the relevant information regarding applicable deep copy rules.
    
    :var objectComp: A tag representing object on which the
    deep copy action need to be performed.
    :var relation: A string representing the name the relation  that need to be deep copied. Valid value can be
    IMANRelation.
    :var objName: A string representing the new name for the new copy.
    of the object represented by otherSideObjectTag. The value for
    the newName will be null if the 'action' is not CopyAsObject or
    ReviseAndRelateToLatest.
    :var operationType: An integer representing the action to be performed on the object represented by
    'otherSideObjectTag'.
    The values for action are:
    CopyAsObjectType = 0,
    CopyAsRefType = 1,
    DontCopyType =2,
    RelateToLatest = 3,
    ReviseAndRelateToLatest = 4
    :var isTargetPrimary: A Boolean representing whether the given item
    revision is a primary object in the relation that need
    to be deep copied.
    :var isRequired: A Boolean representing whether the deep information is from a mandatory deep copy rule configured
    by the administrator or not.
    """
    objectComp: BusinessObject = None
    relation: str = ''
    objName: str = ''
    operationType: int = 0
    isTargetPrimary: bool = False
    isRequired: bool = False


@dataclass
class ScheduleDeliverableData(TcBaseObj):
    """
    A container for a new Schedule Deliverable.
    
    :var schedule: The schedule to contain this deliverable.
    :var deliverableType: The type of the deliverable.
    :var deliverableName: The name of the deliverable.
    :var deliverableReference: A reference to the deliverable.(optional)
    """
    schedule: Schedule = None
    deliverableType: str = ''
    deliverableName: str = ''
    deliverableReference: WorkspaceObject = None


@dataclass
class BillRateContainer(TcBaseObj):
    """
    The container for a BillRate.
    
    :var type: The type: (0-Multiplier, 1-Custom Rate).
    :var name: The name of the bill rate.
    :var rate: The new hourly rate or multiplier in the format *n.nnn (15.3  max).
    :var currency: The ISO 4217 currency rate.
    """
    type: int = 0
    name: str = ''
    rate: str = ''
    currency: str = ''


@dataclass
class StringValContainer(TcBaseObj):
    """
    The container represents a single attribute's value. 
    The name of the attribute is represented by the key field. 
    The data type of the attribute is represented by the type field. The type field can be one of the following;
    char = 0;
    date = 1;
    double = 2;
    float = 3;
    int = 4;
    bool = 5;
    short = 6;
    string = 7;
    ModelObject = 8;
    
    :var key: A string key identifying the attribute.
    :var value: A string representation of the value of the attribute.
    :var type: An integer to help determine the data type of the attribute. Valid values- {
    1=String_type,2=Integer_type,3=Long_type,4=Double_type,5=Float_type,6=Boolean_type,7=Date_type,8=Cal_type }
    """
    key: str = ''
    value: str = ''
    type: int = 0


@dataclass
class TaskCostUpdate(TcBaseObj):
    """
    The request container for fixed cost and costing meta data updates.
    
    :var task: The task which contains or will contain the fixed costs.
    :var newCosts: The information needed to create new fixed costs.
    :var updatedCosts: The information needed to update existing fixed costs.
    :var deletedCosts: A list of fixed costs to delete.
    :var billCode: The bill code for the task. Valid values are determined by the LOV. Valid values are {unassigned,
    General, Management, ProjectMgmt, Sales, Training, Travel, ProductDev, SoftwareDev}
    :var subCode: The bill subcode for the task. Valid values are determined by the LOV. Valid value are following
    groups depending upon billcode.eg. If billcode is General then you can mentioned subcode as either of following set
    (unassigned, Accounting, Clerical, CorpAdmin, IT, Meetings, Other)                    Following are the list of
    billcode and corresponsing valid values for sub code.
    For BillCode 'unassigned' = { unassigned }
    For BillCode 'General' ={ unassigned, Accounting, Clerical, CorpAdmin, IT, Meetings, Other },
    For Bill code Management = { unassigned, Executive, ProjMgmt, Design/Plan, Meetings, Training, Other}
    For BillCode 'ProiectMgmt' ={ unassigned, Management, Meetings, Design/Plan, Training, Teaching, Clerical, Email,
    Other }
    For BillCode 'Sales' ={ unassigned, MajorAccts, General, Admin, Training, Other }
    For BillCode 'Training' ={ unassigned, Billable, Customer1, Customer2, Customer3, Region1, Region2, Region3, Other }
    For BillCode 'Travel' ={ unassigned, Billable, Region1, Region2, Region3, Other }                                  
        For BillCode 'ProductDev' ={ unassigned, Planning, Design, Development, ProcessMgt, Validation,Other}          
                                               For BillCode 'SoftwareDev' ={unassigned, Concept, Defination,
    Development, Introduction, Training, Other }
    :var billingType: The billing type of the task. Valid values are determined by the LOV. Valid value are {
    unassigned, Billable, Billed, Standard, Unbillable}
    :var billRate: The name of the bill rate associated with the task.
    Valid value can be empty string
    :var rate: A tag to the bill rate to associate with this task.
    Valid value can be NULLTAG
    """
    task: ScheduleTask = None
    newCosts: List[FixedCostContainer] = ()
    updatedCosts: List[FixedCostContainer] = ()
    deletedCosts: List[SchedulingFixedCost] = ()
    billCode: str = ''
    subCode: str = ''
    billingType: str = ''
    billRate: str = ''
    rate: BillRate = None


@dataclass
class TaskCostUpdateResponse(TcBaseObj):
    """
    The response container for fixed cost update.
    
    :var updatedTask: The updated task.
    :var newCosts: The list of new fixed costs.
    """
    updatedTask: ScheduleTask = None
    newCosts: List[SchedulingFixedCost] = ()


@dataclass
class TypedAttributeContainer(TcBaseObj):
    """
    A container which is used to update custom properties.
    
    :var type: The object type. Valid values are { 
    ScheduleType,ScheduleRevisionType,ScheduleTaskType,ScheduleTaskRevisionType}
    :var attributes: A collection of updates.(optional)
    """
    type: str = ''
    attributes: List[StringValContainer] = ()


@dataclass
class UpdateTaskCostDataResponse(TcBaseObj):
    """
    The response to the cost update.
    
    :var responses: The collection of individual task cost changes.
    :var serviceData: The ServiceData.
    """
    responses: List[TaskCostUpdateResponse] = ()
    serviceData: ServiceData = None


@dataclass
class CreateBillRateResponse(TcBaseObj):
    """
    The response when BillRates are created.
    
    :var serviceData: The ServiceData.
    :var rates: The added BillRates.
    """
    serviceData: ServiceData = None
    rates: List[BillRate] = ()


@dataclass
class AddMembershipResponse(TcBaseObj):
    """
    The response when schedule members are added.
    
    :var data: The ServiceData.
    :var addedMembers: The added members.
    """
    data: ServiceData = None
    addedMembers: List[ScheduleMember] = ()


@dataclass
class CreateScheduleResponse(TcBaseObj):
    """
    The response when new Schedule is created
    
    :var serviceData: The ServiceData
    :var schedules: The created Schedules
    """
    serviceData: ServiceData = None
    schedules: List[Schedule] = ()



from __future__ import annotations

from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ObjectPropertyPolicy(TcBaseObj):
    """
    System class - needs reverse engineering!
    """
    modifiers: List[Modifiers] = ()
    types: List[PolicyType] = ()
    useRefCount: bool = False


@dataclass
class PolicyProperty(TcBaseObj):
    """
    System class - needs reverse engineering!
    """
    name: str = ''
    modifiers: List[Modifiers] = ()


@dataclass
class PolicyType(TcBaseObj):
    """
    System class - needs reverse engineering!
    """
    name: str = ''
    modifiers: List[Modifiers] = ()
    properties: List[PolicyProperty] = ()


@dataclass
class Modifiers(TcBaseObj):
    """
    System class - needs reverse engineering!
    """
    name: str = ''
    value: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Manufacturing._2020_04.DataManagement import GetProductScopeForProcessResponse
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def getProductScopeForProcess(cls, processLine: List[BusinessObject]) -> GetProductScopeForProcessResponse:
        """
        This service returns a list of product line from BOM (Bill of Material) structure which are related to process
        line with "Fnd0ProcessToScopeRel" relation.
        
        Use cases:
        User selects a process line from the Bill of Proces (BOP) structure and selects "Set/Edit Scope" from context
        menu. The "Set Product Bucket" dialog lists all the product BOMLines related to selected process line.
        """
        return cls.execute_soa_method(
            method_name='getProductScopeForProcess',
            library='Internal-Manufacturing',
            service_date='2020_04',
            service_name='DataManagement',
            params={'processLine': processLine},
            response_cls=GetProductScopeForProcessResponse,
        )



from __future__ import annotations

from tcsoa.gen.StructureManagement._2019_06.VariantManagement import SetBOMVariantRulesResponse2, SetBOMVariantRuleData2, BOMVariantRulesResponse2, ApplyBOMVariantRulesResponse2, BOMVariantRuleContents2
from tcsoa.gen.BusinessObjects import BOMWindow
from tcsoa.gen.StructureManagement._2013_05.VariantManagement import GetBOMVariantRuleInput
from typing import List
from tcsoa.utils import TcService


class VariantManagementService(TcService):

    @classmethod
    def setBOMVariantRules2(cls, setBOMVariantRuleInput: List[SetBOMVariantRuleData2]) -> SetBOMVariantRulesResponse2:
        """
        This operation set the input variant rule to the window and returns the list of variant rule and respective
        window.
        
        Use cases:
        This operation should be used when user want to set values of the option on variant rule.
        
        Exceptions:
        >This operation may raise a Teamcenter::Soa::Server::ServiceException.
        &bull;    214515   The operation has encountered an internal error.
        """
        return cls.execute_soa_method(
            method_name='setBOMVariantRules2',
            library='StructureManagement',
            service_date='2019_06',
            service_name='VariantManagement',
            params={'setBOMVariantRuleInput': setBOMVariantRuleInput},
            response_cls=SetBOMVariantRulesResponse2,
        )

    @classmethod
    def applyBOMVariantRules2(cls, window: BOMWindow, rules: List[BOMVariantRuleContents2]) -> ApplyBOMVariantRulesResponse2:
        """
        The applyBOMVariantRules2 operation applies either given BOM variant rules or Saved Variant Rules to the
        window. BOM Variant rules that contain options having multiple values can be applied to the window. Output will
        be the window and list of BOM variant rules and Saved Variant Rules have been applied to the window.
        
        Use cases:
        This operation will be used when BOM variant rules or Saved Variant Rules needs to be applied on the window.
        
        Exceptions:
        >This operation may raise a Teamcenter::Soa::Server::ServiceException.
        &bull;    214515   The operation has encountered an internal error.
        """
        return cls.execute_soa_method(
            method_name='applyBOMVariantRules2',
            library='StructureManagement',
            service_date='2019_06',
            service_name='VariantManagement',
            params={'window': window, 'rules': rules},
            response_cls=ApplyBOMVariantRulesResponse2,
        )

    @classmethod
    def getBOMVariantRules2(cls, variantRuleInput: List[GetBOMVariantRuleInput]) -> BOMVariantRulesResponse2:
        """
        This operation takes list of windows and its identifier and returns variant rules and saved variant rules
        associated with the window. As part of input in this operation user can provide saved variant rule along with
        saved variant rule action mode. This action indicates add, remove, update or override actions related to the
        saved variant rule. There could be multiple variant rules, associated with window. List of these rules will be
        returned as the output. It also returns list of option and list of values associated with each option. A flag
        in the value list indicates, if window is configured with the particular option value. There could be multiple
        values associated with a single option and there could be multiple saved variant rules associated with a window.
        
        Use cases:
        This operation should be used when user wants to get variant rules associated with window. User may also use it
        to set, unset, override or update saved variant rule based on the window mode.
        
        Exceptions:
        >This operation may raise a Teamcenter::Soa::Server::ServiceException.
        &bull;    214515   The operation has encountered an internal error.
        """
        return cls.execute_soa_method(
            method_name='getBOMVariantRules2',
            library='StructureManagement',
            service_date='2019_06',
            service_name='VariantManagement',
            params={'variantRuleInput': variantRuleInput},
            response_cls=BOMVariantRulesResponse2,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision, Dataset
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FileInfo(TcBaseObj):
    """
    File information
    
    :var clientId: The client id
    :var fileName: The file name
    :var namedReferencedName: The Dataset named reference information.
    :var isText: True means Text format, otherwise it is Binary format.
    :var allowReplace: Allow to replace or not.
    """
    clientId: str = ''
    fileName: str = ''
    namedReferencedName: str = ''
    isText: bool = False
    allowReplace: bool = False


@dataclass
class FileTicketInfo(TcBaseObj):
    """
    File ticket information structure
    
    :var fileInfo: The file information.
    :var ticket: The FMS ticket.
    """
    fileInfo: FileInfo = None
    ticket: str = ''


@dataclass
class GetAdditionalFilesForCheckinInputs(TcBaseObj):
    """
    The structure contains the item revision and file names.
    
    :var itemRevision: The item revision
    :var filenames: The full file path and file names.
    """
    itemRevision: ItemRevision = None
    filenames: List[str] = ()


@dataclass
class GetAdditionalFilesForCheckinOutputs(TcBaseObj):
    """
    The structure contains the item revision and vector of CommitDatasetInfo structure.
    
    :var itemRevision: The item revision
    :var datasetInfos: The dataset information.
    """
    itemRevision: ItemRevision = None
    datasetInfos: List[CommitDatasetInfo] = ()


@dataclass
class GetAdditionalFilesForCheckinOutputsResponse(TcBaseObj):
    """
    The structure contains vector of GetAdditionalFilesForCheckinOutputs and the serviceData.
    
    :var outs: The vector of GetAdditionalFilesForCheckinOutputs.
    :var serviceData: The service data.
    """
    outs: List[GetAdditionalFilesForCheckinOutputs] = ()
    serviceData: ServiceData = None


@dataclass
class GetCheckinModeAndFilesOutputs(TcBaseObj):
    """
    The structure contains the ItemRevision and vector of file names and the check in mode.
    
    :var itemRevision: The ItemRevision business object
    :var filenames: The list of file names that are currently downloaded locally when the ItemRevision was checked out
    :var mode: The CheckIn mode
    """
    itemRevision: ItemRevision = None
    filenames: List[str] = ()
    mode: str = ''


@dataclass
class GetCheckinModeAndFilesOutputsResponse(TcBaseObj):
    """
    The structure contains vector of getCheckinModeAndFilesOutput structure and serviceData.
    
    :var outs: The list of Struct 'GetCheckinModeAndFilesOutputs'
    :var serviceData: The Service Data. Partial errors and failures are updated and returned through this object
    """
    outs: List[GetCheckinModeAndFilesOutputs] = ()
    serviceData: ServiceData = None


@dataclass
class PostCreateInfo(TcBaseObj):
    """
    Post create information
    
    :var itemRevision: The ItemRevision business object that is newly created
    :var commitInfos: The list of 'CommitDatasetInfo' struct
    """
    itemRevision: ItemRevision = None
    commitInfos: List[CommitDatasetInfo] = ()


@dataclass
class PostCreateInputs(TcBaseObj):
    """
    Document Management Post Create Inputs
    
    :var clientId: The client id
    :var itemRevision: The newly created ItemRevision
    :var fileNames: Attached file names
    """
    clientId: str = ''
    itemRevision: ItemRevision = None
    fileNames: List[str] = ()


@dataclass
class PostCreateResponse(TcBaseObj):
    """
    Post Create Response structure
    
    :var output: List of 'PostCreateInfo' struct
    :var serviceData: The Service Data. Partial errors and failures are updated and returned through this object
    """
    output: List[PostCreateInfo] = ()
    serviceData: ServiceData = None


@dataclass
class CommitDatasetInfo(TcBaseObj):
    """
    Commit dataset information
    
    :var dataset: The Dataset created.
    :var dsTypeName: The Dataset type name.
    :var fileTicketInfo: The file ticket information.
    """
    dataset: Dataset = None
    dsTypeName: str = ''
    fileTicketInfo: FileTicketInfo = None



from tcsoa.gen.Internal_Security._2017_12.services import AwLicensingService as imp0
from tcsoa.utils import TcService


class AwLicensingService(TcService):
    getLicensesWithTypes = imp0.getLicensesWithTypes



from __future__ import annotations

from typing import Dict, List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class SupportedHandlerArgumentsInfoOutput(TcBaseObj):
    """
    SupportedHandlerArgumentsInfoOutput structure represents handler Data with dynamic hints.
    
    :var clientId: A unique string used to identify return data elements.
    :var handlerData: Handler Data with dynamic hints (as a JSON string). The following hints are included:
    &bull;    Mandatory Arguments
    &bull;    Optional arguments.
    &bull;    Mutex arguments.
    &bull;    Dependent arguments.
    &bull;    Nullable arguments.
    &bull;    Required one of arguments.
    &bull;    And argument values which allow only particular set of values for ex: list of object types, status,
    relations and LOVs.
    :var additionalData: This is a map that will point to a list of strings. It can be used incase there is a need to
    send any additional data.
    """
    clientId: str = ''
    handlerData: str = ''
    additionalData: HandlerKeyValuePair = None


@dataclass
class SupportedHandlerArgumentsInput(TcBaseObj):
    """
    SupportedHandlerArgumentsInput stucture represent the input handler name and clientId.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements.
    :var handlerName: Name of the handler.
    :var additionalData: This is a map that will point to a list of strings. It can be used incase there is a need to
    send any additional data as input to this operation.
    """
    clientId: str = ''
    handlerName: str = ''
    additionalData: HandlerKeyValuePair = None


@dataclass
class SupportedHandlerArgumentsReponse(TcBaseObj):
    """
    SupportedHandlerArgumentsReponse represents list of SupportedHandlerArgumentsInfoOutput which contains information
    about handlerData and clientId.
    
    :var output: List of SupportedHandlerArgumentsInfoOutput structure which contains handler data info.
    """
    output: List[SupportedHandlerArgumentsInfoOutput] = ()


"""
Structure for passing handler data in form of key and values.
"""
HandlerKeyValuePair = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.ChangeManagement._2015_10.ChangeManagement import DeleteChangeLineageInputData, CreateChangeLineageResponse, CreateChangeLineageInputData
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ChangeManagementService(TcService):

    @classmethod
    def deleteChangeLineage(cls, inputs: List[DeleteChangeLineageInputData]) -> ServiceData:
        """
        This operation deletes the change lineage by deleting the relations between a group of solution items and their
        respective impacted items that are associated with the lineage group. Each DeleteChangeLineageInputData
        structure passed as input will have the clientId to uniquely identify each input, the ChangeNoticeRevision and
        the list of objects for which the lineage has to be deleted. Lineage group is determined for each object and
        all the relations belonging to the same lineage group will be deleted.
        
        Use cases:
        For a ChangeNoticeRevision, change lineage creation is the ability to relate a group of solution items with
        their respective impacted items and designate an associated lineage group. This operation allows deleting
        change lineage.
        """
        return cls.execute_soa_method(
            method_name='deleteChangeLineage',
            library='ChangeManagement',
            service_date='2015_10',
            service_name='ChangeManagement',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def createChangeLineage(cls, input: List[CreateChangeLineageInputData]) -> CreateChangeLineageResponse:
        """
        This operation creates CMSolutionToImpacted relation between all the Solution Items and all the Impacted Items
        for a ChangeNoticeRevision and then assign a group ID to them. To determine the group ID, the
        CMSolutionToImpacted relations are traversed for ChangeNoticeRevision and then find the largest group ID number
        present for this ChangeNoticeRevision in context. The new group ID is the next incremented number. This group
        number is assigned to all the CMSolutionToImpacted relations created for the input data.
        Each CreateChangeLineageInputData input will have new group id assigned for the relations created for that
        group.
        
        Note: Solution Items and Impacted Items are the objects which are attached to ChangeNoticeRevision using
        CMHasSolutionItem and CMHasImpactedItem relation respectively.
        
        Use cases:
        User selects Solution Items and Impacted Items for a ChangeNoticeRevision and create lineage between them.
        """
        return cls.execute_soa_method(
            method_name='createChangeLineage',
            library='ChangeManagement',
            service_date='2015_10',
            service_name='ChangeManagement',
            params={'input': input},
            response_cls=CreateChangeLineageResponse,
        )



from __future__ import annotations

from tcsoa.gen.Classification._2007_01.Classification import ClassificationProperty
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from tcsoa.gen.BusinessObjects import WorkspaceObject
from dataclasses import dataclass


@dataclass
class SaveClassificationObjectsResponse(TcBaseObj):
    """
    Holds the classification objects returned by the saveClassificationObjects operation.
    
    :var clsObjs: A list of created or updated Classification objects.
    :var data: Any failures will be returned in the service data list of partial errors.
    """
    clsObjs: List[ClassificationObjectInfo] = ()
    data: ServiceData = None


@dataclass
class ClassificationObjectInfo(TcBaseObj):
    """
    Structure representing Classification Object details.
    
    :var clsObjUid: UID of Classification object.
    :var instanceId: Alphanumeric ID of the Classification object.
    :var classId: Unique Alphanumeric ID of the Classification class where this object was created.
    :var unitBase: Unit system of measure in which the Classification object is stored in.
    :var wsoId: Reference of the WorkspaceObject (WSO) that is associated by this Classification object. This can be
    empty.
    :var properties: Array of Classification attributes references that store the properties of this Classification
    object.
    :var relationUid: UID of IMAN_classification relation.
    :var lastModDate: Last modified date of the Classified WorkspaceObject.
    """
    clsObjUid: str = ''
    instanceId: str = ''
    classId: str = ''
    unitBase: UnitSystem = None
    wsoId: WorkspaceObject = None
    properties: List[ClassificationProperty] = ()
    relationUid: str = ''
    lastModDate: str = ''


class UnitSystem(Enum):
    """
    US_UNSPECIFIED: Both or no unit system of measure.
    US_METRIC: Metric unit system of measure.
    US_NON_METRIC: Non-metric unit system of measure.
    """
    US_UNSPECIFIED = 'US_UNSPECIFIED'
    US_METRIC = 'US_METRIC'
    US_NON_METRIC = 'US_NON_METRIC'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import WorkspaceObject, ItemRevision, Item
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from typing import Dict


@dataclass
class LDSelectedInputInfo(TcBaseObj):
    """
    Launch definition selected input information.
    
    :var id: The id can be an Item, an ItemRevision business object, a Dataset business object or a markup control
    object.  It is required.  If empty, invalid object error is returned
    :var item: Related item to input id.
    :var itemRev: Related ItemRevision to the input id.
    :var controlObj: Related markup control object (related to the markup through MarkupContextRelation). The control
    object is determined based on the business object constant Fnd0MarkupControlObject set to true.
    :var requestMode: There are 3 possible values for this element. MARKUP, VIEW, EDIT.  Example, MARKUP is for
    View/Markup action.  VIEW and EDIT is for View/Open action on an office dataset business object.
    :var additionalInfo: This is the map of additional information in the form of key value pair strings.  This can be
    empty.
    """
    id: WorkspaceObject = None
    item: Item = None
    itemRev: ItemRevision = None
    controlObj: WorkspaceObject = None
    requestMode: str = ''
    additionalInfo: KeyValueMap = None


@dataclass
class LaunchDefinitionResponse(TcBaseObj):
    """
    Return data for the laucnh definition.
    
    :var xmlLaunchDef: For the ViewMarkup and OfficeOpen operations, the launch definition xml string.
    For the AWOfficeOpen operation, the launch definition file ticket string.
    :var svcData: The Service Data. Partial errors and failures are updated and returned through this object
    """
    xmlLaunchDef: str = ''
    svcData: ServiceData = None


@dataclass
class ServerInfo(TcBaseObj):
    """
    Server Information.
    
    :var protocol: http or iiop (Protocol type for connection to the server). http is for four tier and iiop is for two
    tier deployment
    :var hostPath: The host path
    :var serverMode: Single character. 2 for two tier. 4 for four tier.  This is to support the VIS legacy ITK
    :var useTccs: If client uses Single Sign On(SSO) to connect to server, then this should be set to true. Otherwise,
    false
    :var useSso: If client uses SSO to connect to server, then this should be set to true. Otherwise, false
    :var tccsEnvironment: Teamcenter client communication system (TCCS) environment name
    :var ssoInfo: SSO information
    :var additionalInfo: This is the map of additional information in the form of key value pair strings
    """
    protocol: str = ''
    hostPath: str = ''
    serverMode: str = ''
    useTccs: bool = False
    useSso: bool = False
    tccsEnvironment: str = ''
    ssoInfo: SsoInfo = None
    additionalInfo: KeyValueMap = None


@dataclass
class SessionInfo(TcBaseObj):
    """
    Session information.
    
    :var descriminator: Client Server session discriminator to connect existing specified session.
    :var additionalInfo: This is the map of additional information in the form of key value pair strings.
    """
    descriminator: str = ''
    additionalInfo: KeyValueMap = None


@dataclass
class SsoInfo(TcBaseObj):
    """
    The information for SSO.
    
    :var loginServiceUrl: The login service Uniform resource locator (Url).
    :var appId: The application Id of the Teamcenter server in this Single Sign On(SSO) environment.
    """
    loginServiceUrl: str = ''
    appId: str = ''


@dataclass
class UserAgentDataInfo(TcBaseObj):
    """
    User application information.
    
    :var userApplication: Client who initiates the VVI launch. The VVI file contains the information necessary for the
    stand alone viewer to open the file.
    :var userAppVersion: Version of the client.
    :var additionalInfo: This is the map of additional information in the form of key value pair strings.
    """
    userApplication: str = ''
    userAppVersion: str = ''
    additionalInfo: KeyValueMap = None


"""
Key value map.
"""
KeyValueMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ProcessTextDatasetResponse(TcBaseObj):
    """
    This ProcessTextDatasetResponse structure is for the response of all processTextDataset operations.
    
    :var content: For the "load" action, this string contains the contents of the text file that is a named reference
    in the Dataset. If there is more than one named reference, only the first one is returned.  For "save" action, this
    string is empty.
    :var serviceData: The Service Data that may contain partial errors.
    """
    content: str = ''
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.Cad._2017_05.StructureManagement import GetProductStructureArrangementsResp, ProductStructureArrangementInput
from typing import List
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def getProductStructureArrangements(cls, psArrInput: List[ProductStructureArrangementInput]) -> GetProductStructureArrangementsResp:
        """
        This operation returns arrangement data for given product structures. The product structures are identified in
        one of two ways, through input of either:
        * a "root" PSBOMViewRevision.  For each root structure, a list of any additional substructures from which to
        obtain arrangement data can be provided. 
        * a BOMWindow with list of Fnd0BOMLineLite (LightWeightBOM or LWB) lines of that window.
        This operation does not load objects. It provides Access Manager (AM) data access checks at the database level,
        for any data returned.
        
        For each product structure, a "base" arrangement is created and returned. 
        A base arrangement consists of relative occurrence transformation information for each immediate child
        occurrence of the product structure, along with any overrides of those occurrences that are not in context of
        any arrangement. These are transformation or geometry (jt) overrides. 
        Note that the base arrangement will contain information for all immediate child occurrences. No configuration
        evaluation is performed, e.g. no occurrence effectivity or variant configuration is applied. 
        If no AssemblyArrangements are found for a product structure, the return for that product structure is only the
        base arrangement.
        
        Use cases:
        UseCase: Multilevel structure with overrides on substructure lines
        
        Given the following structure with arrangements defined:
        TopAssembly with arrangements TopDefault, Top0
        ------SubAssembly with arrangements SubDefault, Sub0
        ------------Part1 with transformation override in context of TopAssembly, no arrangement
        ------------Part2 with transformation override in context of SubAssembly, SubDefault arrangement
        ------------Part3 with transformation override in context of SubAssembly, Sub0 arrangement
        The TopDefault arrangement "uses" the SubDefault arrangement on the SubAssembly child.
        The Top0 arrangement "uses" the Sub0 arrangement on the SubAssembly child.
        
        With psArrInput vector containing a singele element which itself  is a vector with the single input of Top
        Assembly PSBOMViewRevision, the returned output vector contains a single element which itself is a vector with
        the single arrangementData structure for TopAssembly. The arrangementData structure contains the following:
        -TopAssembly base arrangement, with:
           * Occurrence data for the immediate child SubAssembly (occurrencePath and relative transformation)
           * Occurrence override data for the immediate child SubAssembly&rsquo;s Part1 transformation override which
        is in context of TopAssembly.
        -TopAssembly arrangement TopDefault, with override data uses SubDefault arrangement (the arrangmentID.)
        -TopAssembly arrangement Top0, with override data uses Sub0 arrangement (the arrangmentID.)
        Note that the ArrangementID is a structure defined to hold the arrangementName and subfileID that identifies
        the specific Arrangement.
        
        With psArrInput vector containing a singele element which itself  is a vector with 2 inputs, Top Assembly and
        SubAssembly PSBOMViewRevisions, the returned output vector contains a single element which itself is a vector
        of size 2 holding the arrangementData for the input as follows.
        TopAssembly:
        -TopAssembly base arrangement, with:
           * Occurrence data for the immediate child SubAssembly (occurrencePath and relative transformation)
           * Occurrence override data for the immediate child SubAssembly&rsquo;s Part1 transformation override which
        is in context of TopAssembly.
        -TopAssembly arrangement TopDefault, with override data uses SubDefault arrangement (the arrangmentID.)
        -TopAssembly arrangement Top0, with override data uses Sub0 arrangement (the arrangmentID.) 
        
        SubAssembly: 
        -SubAssembly base arrangement with:
           * Occurrence data for each of the the immediate children Part1, Part2, Part3 (occurrencePath and relative
        transformation)
        -SubAssembly arrangement SubDefault, with transformation override data for Part2
        -SubAssembly arrangement Sub0, with transformation override data for Part3
        """
        return cls.execute_soa_method(
            method_name='getProductStructureArrangements',
            library='Internal-Cad',
            service_date='2017_05',
            service_name='StructureManagement',
            params={'psArrInput': psArrInput},
            response_cls=GetProductStructureArrangementsResp,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Item, BOMLine, BOMWindow
from tcsoa.gen.Internal.StructureManagement._2007_06.PublishByLink import CreateIDCWindowResponse
from tcsoa.gen.Internal.StructureManagement._2007_06.GlobalAlternate import PreferredGlobalAlternateInput, GlobalAlternateListInput, GlobalAlternateResponse
from tcsoa.gen.Internal.StructureManagement._2007_06.Restructure import MoveNodeParameter, InsertLevelParameter, SplitOccurrenceParameter
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class RestructureService(TcService):

    @classmethod
    def insertLevel(cls, input: List[InsertLevelParameter]) -> ServiceData:
        """
        Insert one item above the selected BOM lines, absolute occurrences, predecessor will be preserved, Incremental
        Change Element will be adjusted.
        
        Note:
        - The operation cannot be invoked in pending edit mode.
        - The selected lines should be in same BOM.
        - The operation will need to be run in incremental change context if any selected line has Incremental Change.
        - When the operation is invoked in incremental context and one of the selected line has REMOVE Incremental
        Change, the line will be skipped.
        - When the operation is invoked in incremental context and one of the selected line has ADD Incremental Change,
        the operation will fail.
        - If any selected line has any absolute occurrence, after insert level operation, the new Appearance Path Node
        of the newly created line will share the same absolute occurrence object with the Appearance Path Node of the
        original line if user has revised the structure before calling the operation.
        
        
        
        Use cases:
        - User creates a new item and adds it as new level above some packed lines by calling the operation with the
        packed lines and the newly created item. After insert level, the new lines are still packed.
        - A structure has predecessor relationship among lines, user adds a new level above some but not all of the
        lines by invoking the operation with an existing item, the item will be added as new level on the selected
        lines and the predecessor relationship will be maintained among the selected lines.
        - Selected lines have some absolute occurrences but not in current context, user invokes the
        
        """
        return cls.execute_soa_method(
            method_name='insertLevel',
            library='Internal-StructureManagement',
            service_date='2007_06',
            service_name='Restructure',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def moveNode(cls, input: List[MoveNodeParameter]) -> ServiceData:
        """
        Move pending-cut BOMLines to the selected BOMLine, absolute occurrences, predecessor relationship will be
        preserved, Incremental Change Element and modular variants data will be adjusted.
        
        Note:
        - The operation cannot be invoked in pending edit mode.
        - The operation will need to be run in incremental change context if any selected line has incremental change.
        - When the operation is invoked in incremental context and one of the selected line has REMOVE incremental
        change, the line will be skipped.
        - When the operation is invoked in incremental context and one of the selected line has ADD incremental change,
        the operation will fail.
        - Incremental Change Elements of the source line will be carried over to the new line.
        - If reference designator validation is enabled, reference designator value will be wiped out for the new lines
        if the original reference designator value is no longer valid.
        - For modular variants, if the line is moved out of context, variant condition will be invalid, the operation
        will wipe out the variant condition.
        - If any selected line has any absolute occurrence, after this operation, the new appearance path node of the
        newly created line will share the same absolute occurrence object with the Appearance Path Node of the original
        line if user has revised the structure before calling the operation.
        - If a line is moved out of the context of its absolute occurrence, the absolute occurrence will be lost.
        - When preference TC_move_to_special_case_detect is enabled, the operation will check a special case that the
        operation cannot support and will abort the operation if the case exists. The special case is that if we move a
        line which carries absolute occurrences, to a line, the occurrence thread of which line has multiple lines in
        the structure.
        
        
        
        Use cases:
        If user has the following structure
        
        Root
        +----sub0
        +----sub1
        +-------------------------leaf1 ( absocc1 in context of root)
        +-------------------------sub11
        +-------------------------leaf2(absocc2 in context of sub1)
        
        User moves leaf1 and leaf2 to sub0 to get:
        
        Root
        +----sub0
        +-------------------------leaf1 ( absocc1 in context of root)
        +-------------------------leaf2
        +----sub1
        +-------------------------sub11
        """
        return cls.execute_soa_method(
            method_name='moveNode',
            library='Internal-StructureManagement',
            service_date='2007_06',
            service_name='Restructure',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def removeLevel(cls, bomLines: List[BOMLine]) -> ServiceData:
        """
        Remove the selected BOMLines and attach the children of the removed lines to the parent of the selected lines,
        absolute occurrences, predecessor relationship will be preserved, Incremental Change Element and modular
        variants will be adjusted.
        
        Note:
        - The operation cannot be invoked in pending edit mode.
        - The operation cannot be used for root lines.
        - The operation will need to be run in incremental change context if any selected line has Incremental Change.
        - When the operation is invoked in incremental context and one of the selected line has REMOVE Incremental
        Change, the line will be skipped.
        - When the operation is invoked in incremental context and one of the selected line has ADD Incremental Change,
        the operation will fail.
        - The final quantity of the new occurrence will be the calculated based on the quantity of the selected line
        and the child.
        - AppearanceGroupBOPLine will not participate in the operation
        - If reference designator validation is enabled, reference designator value will be wiped out for the new lines.
        - Options on the selected lines will be copied to the parent and variant conditions of the selected lines and
        their children will be merged together for both modular variants and legacy variants.
        - Absolute occurrences in context of the removed lines will be lost.
        - If any selected line has any absolute occurrence, after insert level operation, the new Appearance Path Node
        of the newly created line will share the same absolute occurrence object with the Appearance Path Node of the
        original line if user has revised the structure before calling the operation.
        
        
        
        Use cases:
        If user has the following structure
        
        Root (optionRoot)
        +----toBeRemoved ( optionA, quantity=4, load if optionRoot = 1)
        +-------------------------leaf ( absocc1 in context of toBeRemoved, aboscc2 in context of root, quantity=5,
        load if optionA=A)
        
        User removes level of toBeRemoved by invoking the operation and get the following structure:
        
        Root (optionRoot, optionA)
        +---------leaf (aboscc2 in context of root, quantity=20, load if optionA=A && optionRoot=1)
        """
        return cls.execute_soa_method(
            method_name='removeLevel',
            library='Internal-StructureManagement',
            service_date='2007_06',
            service_name='Restructure',
            params={'bomLines': bomLines},
            response_cls=ServiceData,
        )

    @classmethod
    def splitOccurrence(cls, input: List[SplitOccurrenceParameter]) -> ServiceData:
        """
        Split certain quantity of occurrences from the selected BOMLines, absolute occurrences will be preserved.
        
        Note:
        - The operation cannot be invoked in pending edit mode.
        - The operation cannot be used for root lines.
        - The operation will need to be run in incremental change context if any selected line has incremental change.
        - When the operation is invoked in incremental context and one of the selected line has ADD incremental change,
        the operation will fail.
        - Objects of types specified by preference BOM_objects_excluded_from_split will not perform the operation.
        - If reference designator validation is enabled, reference designator value will be wiped out for the new lines.
        - If any selected line has any absolute occurrence, after insert level operation, the new Appearance Path Node
        of the newly created line will have a copy of the absolute occurrence object for the Appearance Path Node of
        the original line.
        - Packed lines will not be supported.
        - Quantity should be bigger than zero and less than the total of the original quantity.
        
        
        
        Use cases:
        User has a line which has absolute occurrence data, quantity for the line is 10.5, user can split the
        occurrence by quantity 0.5 by calling the operation and get two lines, one has quantity 0.5 and the other one
        has quantity 10.0, both lines will have the same absolute occurrence data.
        """
        return cls.execute_soa_method(
            method_name='splitOccurrence',
            library='Internal-StructureManagement',
            service_date='2007_06',
            service_name='Restructure',
            params={'input': input},
            response_cls=ServiceData,
        )


class GlobalAlternateService(TcService):

    @classmethod
    def listGlobalAlternates(cls, items: List[Item]) -> GlobalAlternateResponse:
        """
        This operation gets the lists of Global Alternates for the selected set of items. A global alternate is
        interchangeable with another item in all circumstances, regardless of where the other item is used in the
        product structure.
        
        Use cases:
        In order to list all the global alternates of the selected item, the user can select an item or a BOMLine in
        the product structure, and then call 'listGlobalAlternates' operation to get the global alternates for the
        selected item.
        """
        return cls.execute_soa_method(
            method_name='listGlobalAlternates',
            library='Internal-StructureManagement',
            service_date='2007_06',
            service_name='GlobalAlternate',
            params={'items': items},
            response_cls=GlobalAlternateResponse,
        )

    @classmethod
    def removeRelatedGlobalAlternates(cls, input: List[GlobalAlternateListInput]) -> GlobalAlternateResponse:
        """
        Remove selected global alternates from the list of global alternates of the selected item. A global alternate
        is interchangeable with another item in all circumstances, regardless of where the other item is used in the
        product structure.
        
        Use cases:
        In order to remove one or more global alternates from the global alternate list of the selected item, the user
        can call 'removeRelatedGlobalAlternates' operation and pass in the selected item and a list of selected global
        alternates to be removed from the list.
        """
        return cls.execute_soa_method(
            method_name='removeRelatedGlobalAlternates',
            library='Internal-StructureManagement',
            service_date='2007_06',
            service_name='GlobalAlternate',
            params={'input': input},
            response_cls=GlobalAlternateResponse,
        )

    @classmethod
    def setPreferredGlobalAlternate(cls, input: List[PreferredGlobalAlternateInput]) -> ServiceData:
        """
        Make the specified global alternate the preferred global alternate for the specified item, or remove the
        preferred designation from a global alternate for the specified item.
        
        Use cases:
        - The user calls 'setPreferredGlobalAlternates' operation to set preferred global alternate, which designates a
        nonpreffered global alternate as the preferred one.
        - The user calls 'setPreferredGlobalAlternates' operation to unset preferred global alternate, which removes
        the preffered designation from a global alternate.
        
        """
        return cls.execute_soa_method(
            method_name='setPreferredGlobalAlternate',
            library='Internal-StructureManagement',
            service_date='2007_06',
            service_name='GlobalAlternate',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def addRelatedGlobalAlternates(cls, input: List[GlobalAlternateListInput]) -> GlobalAlternateResponse:
        """
        This operation adds specified items to the list of global alternates of the selected item. A global alternate
        is interchangeable with another item in all circumstances, regardless of where the other item is used in the
        product structure.
        
        Use cases:
        In order to add one or more global alternates to the selected item, the user can call
        'addRelatedGlobalAlternates' operation and pass in the selected item and a list of items to be defined as
        global alternates of the selected item.
        """
        return cls.execute_soa_method(
            method_name='addRelatedGlobalAlternates',
            library='Internal-StructureManagement',
            service_date='2007_06',
            service_name='GlobalAlternate',
            params={'input': input},
            response_cls=GlobalAlternateResponse,
        )


class RedliningService(TcService):

    @classmethod
    def revertAllPendingEdits(cls, windows: List[BOMWindow]) -> ServiceData:
        """
        Resets those properties on all BOMLines of the specified windows that have pended edits, back to their database
        values. Also reverses the structure modifications (additions/removals) done to all BOMLines, back to pre-edited
        state. The functionality is only applicable when editing with option to mark pending edits enabled.
        
        Use cases:
        The user must first have established a single-structure editing session in a BOMWindow, with the ability to
        create pending edits enabled. If the user wants to remove all pending edits from this session, this method will
        restore the edited structure to the state that existed at the start of the session.
        """
        return cls.execute_soa_method(
            method_name='revertAllPendingEdits',
            library='Internal-StructureManagement',
            service_date='2007_06',
            service_name='Redlining',
            params={'windows': windows},
            response_cls=ServiceData,
        )

    @classmethod
    def revertPendingEdits(cls, bomLines: List[BOMLine]) -> ServiceData:
        """
        Resets those properties on the specified BOMLines that have pended edits, back to their database values. Also
        reverses the structure modifications (additions/removals) done to the specified BOMLines, back to pre-edited
        state. The functionality is only applicable when editing with option to mark pending edits enabled.
        
        Use cases:
        The user must first have established a single-structure editing session in a BOMWindow, with the ability to
        create pending edits enabled. If the user wants to remove pending edits from specific BOMLines, this method
        will restore the edited BOMLine to the status that existed at the start of the session.
        """
        return cls.execute_soa_method(
            method_name='revertPendingEdits',
            library='Internal-StructureManagement',
            service_date='2007_06',
            service_name='Redlining',
            params={'bomLines': bomLines},
            response_cls=ServiceData,
        )


class PublishByLinkService(TcService):

    @classmethod
    def createIDCWindowForDesignAsm(cls, comps: List[BusinessObject]) -> CreateIDCWindowResponse:
        """
        Creates IDCWindow for BOMLines or ItemRevisions. The BOMLine or ItemRevision is used to find Text Dataset
        attached through Rendering relation.  The IDCWindow is created with 'BOMWRITER' as context. Top BOMLine of
        BOMWindow is created with Text named reference of the Dataset.
        
        Use cases:
        Create IDWindow with PLMXML file for visualization purpose.
        """
        return cls.execute_soa_method(
            method_name='createIDCWindowForDesignAsm',
            library='Internal-StructureManagement',
            service_date='2007_06',
            service_name='PublishByLink',
            params={'comps': comps},
            response_cls=CreateIDCWindowResponse,
        )



from __future__ import annotations

from tcsoa.gen.Query._2006_03.SavedQuery import ExecuteSavedQueryResponse, DescribeSavedQueriesResponse, GetSavedQueriesResponse
from tcsoa.gen.BusinessObjects import ImanQuery
from typing import List
from tcsoa.utils import TcService


class SavedQueryService(TcService):

    @classmethod
    def getSavedQueries(cls) -> GetSavedQueriesResponse:
        """
        Gets a list of all saved queries with query, query name, and query description information.
        
        Use cases:
        The user can open the search view and can select a query from the Change Search dialog which shows all
        available saved queries.
        The user can open the Query Builder to load all the saved queries, and then do the modification, deletion, and
        creation.
        """
        return cls.execute_soa_method(
            method_name='getSavedQueries',
            library='Query',
            service_date='2006_03',
            service_name='SavedQuery',
            params={},
            response_cls=GetSavedQueriesResponse,
        )

    @classmethod
    def describeSavedQueries(cls, queries: List[ImanQuery]) -> DescribeSavedQueriesResponse:
        """
        Returns the description of each of the input saved queries.The description for each query includes all the
        clause information (the attribute name, entry name, operation for each clause, the math operation for each
        clause, the ListOfValues for related clause if it has, and the attribute type).
        
        Use cases:
        User can get the description for queries by this service and then can show the details in search view so that
        user can execute the query.
        
        User can get the description for queries by this service and then show the details in query builder so that
        user can see the definition for the query or update the query.
        
        User can get the description for queries by this service and then use it to get the saved searches.
        
        User can get the description for queries by this service and then use it to get the search history.
        """
        return cls.execute_soa_method(
            method_name='describeSavedQueries',
            library='Query',
            service_date='2006_03',
            service_name='SavedQuery',
            params={'queries': queries},
            response_cls=DescribeSavedQueriesResponse,
        )

    @classmethod
    def executeSavedQuery(cls, query: ImanQuery, entries: List[str], values: List[str], limit: int) -> ExecuteSavedQueryResponse:
        """
        Executes a single saved query by input query with entries and values. If the returned result number is larger
        than the input limit(when limit > 0), then only the input limit result number objects will be returned;
        otherwise all results will be returned. The number of objects found is also returned; it may be larger than the
        limit number.
        
        Use cases:
        The user opens the search view, selects a query from the system defined queries or user defined queries, then
        fills in some input criteria, clicks the Execute button to run this query. The result objects will be returned
        in the search result view. If the total result objects number is larger than the limit number which is used to
        prevent loading too many objects in memory considering the performance issue(when limit > 0), then only return
        the limit number result objects for the query. The total search result objects number is displayed in the
        search result view.
        """
        return cls.execute_soa_method(
            method_name='executeSavedQuery',
            library='Query',
            service_date='2006_03',
            service_name='SavedQuery',
            params={'query': query, 'entries': entries, 'values': values, 'limit': limit},
            response_cls=ExecuteSavedQueryResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, BOMLine
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetPrimariesOfInStructureAssociationInfo(TcBaseObj):
    """
    Input structure for GetPrimariesOfInStructureAssociationInfo operation
    
    :var secondary: secondary object/BOMLine business object
    :var contextBOMLine: Context object of association. This is generally the parent line of the primary object and is
    an optional parameter.
    :var associationType: the created association type e.g. ConnectTo, ImplementedBy, RealizedBy, DeviceToConnector,
    RoutedBy, SignalToSource, SignalToTarget, SignalToTransmitter, ProcessVariable, RedundantSignal.
    """
    secondary: BusinessObject = None
    contextBOMLine: BOMLine = None
    associationType: str = ''


@dataclass
class GetPrimariesOfInStructureAssociationResponse(TcBaseObj):
    """
    Return structure for GetPrimariesOfInStructureAssociationResponse  operation
    
    :var serviceData: The ServiceData structure is used to return the primary BOMLine business objects and partial
    errors if any occurring in the operation.
    :var primariesInfo: Context object of association. This is generally the parent line of the primary object.
    """
    serviceData: ServiceData = None
    primariesInfo: List[PrimariesOfInStructureAssociation] = ()


@dataclass
class GetSecondariesOfInStructureAssociationInfo(TcBaseObj):
    """
    Input structure for GetSecondariesOfInStructureAssociationInfo operation
    
    :var primaryBOMLine: primary BOMLine
    :var contextBOMLine: Context BOMLine business object of association (optional)
    :var associationType: Association type between the primary and secondary BOMLine business objects.
    """
    primaryBOMLine: BOMLine = None
    contextBOMLine: BOMLine = None
    associationType: str = ''


@dataclass
class GetSecondariesOfInStructureAssociationResponse(TcBaseObj):
    """
    Return structure for GetSecondariesOfInStructureAssociationResponse  operation
    
    :var serviceData: Secondary BOMLine business objects and Partial errors if any
    :var secondariesInfo: primary BOMLine and association type.
    """
    serviceData: ServiceData = None
    secondariesInfo: List[SecondariesOfInStructureAssociation] = ()


@dataclass
class InStructureAssociationInfo(TcBaseObj):
    """
    Input structure for InstructureAssociationInfo operation
    
    :var primaryBOMLine: The primary object of association
    :var contextBOMLine: The context in which the association of the specified type between primaryBOMLine and
    secondaries is valid.
    :var secondaries: secondary objects of association.
    :var associationType: association type to be created
    """
    primaryBOMLine: BOMLine = None
    contextBOMLine: BOMLine = None
    secondaries: List[BusinessObject] = ()
    associationType: str = ''


@dataclass
class PrimariesOfInStructureAssociation(TcBaseObj):
    """
    Output structure for GetPrimariesOfInStructureAssociationInfo operation
    
    :var secondary: secondary object for which the primaries has been returned
    :var associationType: association type to be created
    :var primaryBOMLine: Primary object associated with the secondary
    """
    secondary: BusinessObject = None
    associationType: str = ''
    primaryBOMLine: BOMLine = None


@dataclass
class RemoveInStructureAssociationsInfo(TcBaseObj):
    """
    Input structure for removeInStructureAssociations operation
    
    :var primaryBOMLine: primary BOMLine object reference of association
    :var contextBOMLine: Context BOMLine of association (optional)
    :var secondaries: Secondary BOMLines to be disassociated from the primary BOMLine.
    :var associationType: Association type between primary and secondary BOMLines.
    """
    primaryBOMLine: BOMLine = None
    contextBOMLine: BOMLine = None
    secondaries: List[BusinessObject] = ()
    associationType: str = ''


@dataclass
class RemoveInStructureAssociationsResponse(TcBaseObj):
    """
    Return structure for GetSecondariesOfInStructureAssociationResponse  operation
    
    :var serviceData: The updated objects and partial errors if any.
    """
    serviceData: ServiceData = None


@dataclass
class SecondariesOfInStructureAssociation(TcBaseObj):
    """
    Output structure for GetSecondariesOfInStructureAssociationInfo operation
    
    :var primaryBOMLine: Primary BOMLine associated with the secondary.
    :var associationType: association type to be created
    :var secondaries: Secondary BOMLines associated with the primaries
    """
    primaryBOMLine: BOMLine = None
    associationType: str = ''
    secondaries: List[BusinessObject] = ()


@dataclass
class CreateInStructureAssociationResponse(TcBaseObj):
    """
    Return structure for createInStructureAssociations  operation
    
    :var serviceData: The ServiceData structure is used to return the updated objects and contains partial errors, if
    any, occurring in the operation.
    """
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from typing import Dict


@dataclass
class MfgImportFromBriefcaseResponse(TcBaseObj):
    """
    MfgImportFromBriefcaseResponse structure defines the response from importFromBriefcase operation. It contains FMS
    ticket of the log file, error file, and partial errors and objects that are imported.
    
    :var logFileFMSTicket: FMS ticket of the import log file, which can be used to download the import log file from
    server to client.
    :var errorFileFMSTicket: FMS ticket of the import error file, which can be used to download the import error file
    from server to client.
    :var serviceData: Service data containing the list of created or modified objects and also the partial errors in
    case of failure conditions.
    """
    logFileFMSTicket: str = ''
    errorFileFMSTicket: str = ''
    serviceData: ServiceData = None


"""
Map (string, string) containing names and values.
"""
NamesAndValues = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.StructureManagement._2012_10.Structure import CutItemParam
from tcsoa.gen.StructureManagement._2012_02.StructureVerification import EquivalentLines
from tcsoa.gen.StructureManagement._2012_10.StructureVerification import GetAssignmentComparisonDetailsResponse, StringToPartialMatchCriteria, GetComparisonSummariesResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class StructureService(TcService):

    @classmethod
    def cutItems(cls, input: List[CutItemParam]) -> ServiceData:
        """
        Cut lines of the selected Item Revisions in a window and delete the Item Revisions.
        If the selected Item Revision is the last revision of the Item , then the Item will be deleted.
        
        Use cases:
        Use case1: Simple Cut
        A user wants to delete some Item Revisions in a structure which are not referenced anywhere else. The user
        calls the operation with the BOMWindow and the Item Revisions. The BOMLine for the selected Item Revisions will
        be removed from the structure and the Item Revisions will be deleted.
        
        Use case2: Cut with partial errors
        A user wants to delete some Item Revisions in a structure which are referenced in My Teamcenter and lines of
        some of the Item Revisions are in released structure. The user calls the operation with the BOMWindow and the
        Item revisions. The Item Revisions for the lines that are in released structure will fail to be deleted but the
        other Item Revisions will be deleted and the entries in My Teamcenter will also be removed.
        
        Use case3: Cut the last revision of the item
        A user wants to delete an Item Revision, which is the last revision of the Item. The user calls the operation
        with the BOMWindow and the Item Revision. The BOMLine for the selected Item Revision will be removed from the
        structure and the Item will be deleted
        
        """
        return cls.execute_soa_method(
            method_name='cutItems',
            library='StructureManagement',
            service_date='2012_10',
            service_name='Structure',
            params={'input': input},
            response_cls=ServiceData,
        )


class StructureVerificationService(TcService):

    @classmethod
    def getAssignmentComparisonDetails(cls, equivalentObjects: List[EquivalentLines], partialMatchCriteria: List[StringToPartialMatchCriteria]) -> GetAssignmentComparisonDetailsResponse:
        """
        This operation returns the details of any differences between assignments for the supplied source and target
        BOMLine objects. Assignments can be parts or tools, manual or resolved by logical assignments. The operation
        takes the source and target BOMLine objects and compares their assignments according to their types - manual
        assignments are compared with manual, resolved with resolved, parts with parts and tools with tools. The source
        and target assignments are returned by this operation in the form of a table that is created by the output
        structures.
        """
        return cls.execute_soa_method(
            method_name='getAssignmentComparisonDetails',
            library='StructureManagement',
            service_date='2012_10',
            service_name='StructureVerification',
            params={'equivalentObjects': equivalentObjects, 'partialMatchCriteria': partialMatchCriteria},
            response_cls=GetAssignmentComparisonDetailsResponse,
        )

    @classmethod
    def getComparisonSummaries(cls, equivalentObjects: List[EquivalentLines], partialMatchCriteria: List[StringToPartialMatchCriteria]) -> GetComparisonSummariesResponse:
        """
        This operation retrieves comparison summaries for supplied extensions on the provided equivalent sets of
        objects. The source objects in each set can be BOMLines, Cpd0DesignElements or Cpd0DesignFeatures. The target
        objects in each set can be only BOMLines. For each received extension on each received equivalent set it
        performs comparison of objects in the equivalent set for this extension according to received criteria. For
        each such comparison only the result is returned - a flag indicating whether the input objects were determined
        different or not. This operation returns comparison results for any number of extensions simultaneously.
        """
        return cls.execute_soa_method(
            method_name='getComparisonSummaries',
            library='StructureManagement',
            service_date='2012_10',
            service_name='StructureVerification',
            params={'equivalentObjects': equivalentObjects, 'partialMatchCriteria': partialMatchCriteria},
            response_cls=GetComparisonSummariesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset, Item, AbsOccurrence, BusinessObject, AssemblyArrangement, RevisionRule, PSOccurrenceThread, BOMLine, Folder, PSBOMView, VariantRule, ItemRevision, CFMOverrideEntry, BOMWindow
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ExpandPSAllLevelsInfo(TcBaseObj):
    """
    Contains parent BOMLines to expand and an exclude filter to specify children to exclude.
    
    :var parentBomLines: List of parent bom lines that needs to be expanded
    :var excludeFilter: Filter to exclude the type of BOMLines.
    Valid values are: None -- Returns all information about the structure.
    ExcludeOverridden -- Excludes structure or property values that are removed by AbsOccs subsititution.
    ExcludeICHistory -- Excludes structure (or property values) that are configured out by ICs.
    ExcludeGDEs -- Excludes lines that are GDEOccurrences.
    ExcludeNonImanItemLines -- Excludes any lines that are not ImanItemLines.
    """
    parentBomLines: List[BOMLine] = ()
    excludeFilter: BOMLineFilter = None


@dataclass
class ExpandPSAllLevelsOutput(TcBaseObj):
    """
    Contains ExpandPSParentData corresponding to the parent and a list of ExpandPSData of the children
    
    :var parent: ExpandPSParentData member
    :var children: List of ExpandPSData children found for this parent.
    """
    parent: ExpandPSParentData = None
    children: List[ExpandPSData] = ()


@dataclass
class ExpandPSAllLevelsPref(TcBaseObj):
    """
    More than one filtering criteria can be specified using this which is nothing but a list of  RelationAndTypesFilter
    
    :var expItemRev: Flag to check for expanding the item revision further
    :var info: List of the relation name and the types of objects of the given relation to return along with the
    children
    """
    expItemRev: bool = False
    info: List[RelationAndTypesFilter] = ()


@dataclass
class ExpandPSAllLevelsResponse(TcBaseObj):
    """
    A list of ExpandPSAllLevelsOutput so that a set of parent bom lines can be expanded.
    
    :var output: List of ExpandPSAllLevelsOutput which contains ExpandPSParentData and list of ExpandPSData
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error information.
    For this service, all objects are returned to the plain objects group.
    Error information will also be returned.
    """
    output: List[ExpandPSAllLevelsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ExpandPSData(TcBaseObj):
    """
    Through this structure, the child bom line , the item revision of the bom line and the datasets attached to the bom
    line item  revision are returned.
    
    :var bomLine: BOMline object reference of the children
    :var itemRevOfBOMLine: ItemRevision object reference of children
    :var datasets: List of Dataset object references attached to children
    """
    bomLine: BOMLine = None
    itemRevOfBOMLine: ItemRevision = None
    datasets: List[Dataset] = ()


@dataclass
class ExpandPSOneLevelInfo(TcBaseObj):
    """
    Contains parent BOMLines to expand and an exclude filter to specify children to exclude.
    
    :var parentBomLines: List of parent bom lines that needs to be expanded
    :var excludeFilter: Filter to exclude the type of BOMLines.
    Valid values are: None -- Returns all information about the structure.
    ExcludeOverridden -- Excludes structure or property values that are removed by AbsOccs subsititution.
    ExcludeICHistory -- Excludes structure that are configured out by ICs.
    ExcludeGDEs -- Excludes lines that are GDEOccurrences.
    ExcludeNonImanItemLines -- Excludes any lines that are not ImanItemLines.
    """
    parentBomLines: List[BOMLine] = ()
    excludeFilter: BOMLineFilter = None


@dataclass
class ExpandPSOneLevelOutput(TcBaseObj):
    """
    Structure containing ExpandPSParentData corresponding to the parent and a list of ExpandPSData of the children
    
    :var parent: ExpandPSParentData member
    :var children: List of ExpandPSData children found for this parent.
    """
    parent: ExpandPSParentData = None
    children: List[ExpandPSData] = ()


@dataclass
class ExpandPSOneLevelPref(TcBaseObj):
    """
    More than one filtering criteria can be specified using this which is nothing but a list of  RelationAndTypesFilter
    
    :var expItemRev: Flag to check for expanding the item revision further
    :var info: List of RelationAndTypesFilters that contain the relation names and the types of objects of the given
    relation.
    """
    expItemRev: bool = False
    info: List[RelationAndTypesFilter] = ()


@dataclass
class ExpandPSOneLevelResponse(TcBaseObj):
    """
    A ExpandPSOneLevelResponse containing the return for this operation.
    
    :var output: List of ExpandPSOneLevelOutput which contains ExpandPSParentData and list of ExpandPSData
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error information.
    For this service, all objects are returned to the plain objects group. Error information will also be returned.
    """
    output: List[ExpandPSOneLevelOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ExpandPSParentData(TcBaseObj):
    """
    Through this structure, the parent bom line , the item revision of the bom line and the datasets attached to the
    bom line item revision are returned.
    
    :var bomLine: BOMline object reference of the parent
    :var itemRevOfBOMLine: ItemRevision object reference of parent
    :var parentDatasets: List of Dataset object references attached to parent
    """
    bomLine: BOMLine = None
    itemRevOfBOMLine: ItemRevision = None
    parentDatasets: List[Dataset] = ()


@dataclass
class AbsOccAttachment(TcBaseObj):
    """
    Contains a dataset object reference with the JT override data and a relationTypeName to relate AbsOccData to
    Dataset.
    
    :var dataset: Dataset object reference with the JT override data
    :var relationTypeName: Relation/property to relate AbsOccData to Dataset
    """
    dataset: Dataset = None
    relationTypeName: str = ''


@dataclass
class AssemblyArrangementInfo(TcBaseObj):
    """
    Structure with arrangement qualified override information for the occurrence.
    
    :var clientId: A unique string supplied by the caller.  This ID is used to identify return data elements and
    partial errors associated with this input structure.  If 'clientId' is not provided then it can be difficult to
    align the input with any returned errors.
    :var arrangement: The assembly arrangement object reference.  This input can be null for create, but it is required
    for update.
    :var name: The name of the arrangement.
    :var isDefault: Flag to specify whether 'arrangement' is the default arrangement.
    :var description: The arrangement description.
    :var externalUid: The UID required for create.
    :var absOccInfo: List of absolute occurrence information for the BOM view revision qualified overrides.
    """
    clientId: str = ''
    arrangement: AssemblyArrangement = None
    name: str = ''
    isDefault: bool = False
    description: str = ''
    externalUid: str = ''
    absOccInfo: List[AbsOccInfo] = ()


@dataclass
class GetRevisionRulesResponse(TcBaseObj):
    """
    Contains the response for getRevisionRules operation.
    
    :var output: List of RevisionRuleInfo which contains Revision rule, configure attribute status along with override
    information
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error
    """
    output: List[RevisionRuleInfo] = ()
    serviceData: ServiceData = None


@dataclass
class GetVariantRulesResponse(TcBaseObj):
    """
    Contains the response for getVariantRules operation.
    
    :var inputItemRevToVarRules: Holds map of itemRevision to list of VariantRules.
    :var serviceData: The ServiceData
    """
    inputItemRevToVarRules: ItemRevisionToVariantRulesMap = None
    serviceData: ServiceData = None


@dataclass
class AbsOccDataInfo(TcBaseObj):
    """
    Contains list of AttributesInfo for overrides to set, list of attribute names to unset/remove the overrides, a
    boolean flag, occTransform which contains the positioning information for the occurrence,
    list of note information for the occurrence, list of AbsOccAttachment for the attachments, list of the
    AbsOccAttachment to unattach,
    client id of the used arrangement for this absolute occurrence and a reference of used arrangement for this
    absolute occurrence.
    
    :var overridesToSet: List of AttributesInfo for overrides to set
    :var overridesToRemove: List of attribute names to unset/remove the overrides for on the occurrence, for example to
    remove a transform override, add the attribute name for the transform to this list.
    :var asRequired: Used to the set the quantity as required occurrence flag
    :var occTransform: Positioning information for the occurrence
    :var occNotes: List of note information for the occurrence
    :var attachments: List of AbsOccAttachment for the attachments
    :var attachmentsToUnattach: List of the AbsOccAttachment to unattach
    :var clientIdOfUsedArrangement: Client id of the used arrangement for this absolute occurrence, if arrangement is
    yet to be created
    :var usedArr: Reference of used arrangement for this absolute occurrence if arrangement has already been created
    """
    overridesToSet: List[AttributesInfo] = ()
    overridesToRemove: List[str] = ()
    asRequired: bool = False
    occTransform: List[float] = ()
    occNotes: List[OccNote] = ()
    attachments: List[AbsOccAttachment] = ()
    attachmentsToUnattach: List[AbsOccAttachment] = ()
    clientIdOfUsedArrangement: str = ''
    usedArr: AssemblyArrangement = None


@dataclass
class AbsOccInfo(TcBaseObj):
    """
    Attribute information for the occurrence
    
    :var clientId: Identifier that helps the client track the object(s) created
    :var absOcc: AbsOccurrence object reference , may be null for create
    :var cadOccIdPath: List of IDs of the cad occurrences
    :var absOccData: Member of AbsOccDataInfo
    """
    clientId: str = ''
    absOcc: AbsOccurrence = None
    cadOccIdPath: List[str] = ()
    absOccData: AbsOccDataInfo = None


@dataclass
class OccNote(TcBaseObj):
    """
    Contains note type and note text information for the occurrence note.
    
    :var noteType: The type of the occurrence note to set.
    :var noteText: The text for the occurrence note.
    """
    noteType: str = ''
    noteText: str = ''


@dataclass
class OverrideInfo(TcBaseObj):
    """
    This contains information about the override RevisionRule Entry.
    
    :var ruleEntry: Refers to the CFMOverrideEntry of RevisionRule object.
    :var folder: Refers to the Folder of override rule entry of RevisionRule object.
    """
    ruleEntry: CFMOverrideEntry = None
    folder: Folder = None


@dataclass
class AttributesInfo(TcBaseObj):
    """
    Name and value data to be set as attributes on the related object.
    
    :var name: The attribute name.
    :var value: The value to set for the attribute.
    """
    name: str = ''
    value: str = ''


@dataclass
class RelOccInfo(TcBaseObj):
    """
    Contains information about the relative occurrence.
    
    :var attrsToSet: Name and value pairs for the attribute information to set or update on the occurrence specified in
    the form of BOM line property names.  For example, the BOM line occurrence name property could be specified with
    the 'attrsToSet' 'name' as bl_occurrence_name and the 'value' as the occurrence name.
    :var asRequired: Flag to specify that the quantity is as required.  The default is FALSE.
    :var occTransform: Positioning information for the occurrence.  This needs to be ordered in the standard matrix
    format.
    :var occNotes: Note information for the occurrence.
    """
    attrsToSet: List[AttributesInfo] = ()
    asRequired: bool = False
    occTransform: List[float] = ()
    occNotes: List[OccNote] = ()


@dataclass
class RelationAndTypesFilter(TcBaseObj):
    """
    This consists of a string which indicates the relation name and a list of strings which indicate the object types.
    An object that falls under these input criteria is returned along with the children.
    
    :var relationName: Relation name
    :var relationTypeNames: List of the relation name types
    """
    relationName: str = ''
    relationTypeNames: List[str] = ()


@dataclass
class RelativeStructureChildInfo(TcBaseObj):
    """
    Contains clientId, cadOccId, an item revision and occurrence information structure.
    
    :var clientId: Identifier that helps the client track the object(s) created
    :var cadOccId: This is the CAD occurrence id or PSOccurrenceThread uid to uniquely identify the occurrence under a
    particular context Item Revision, can be null for create.
    A valid cadOccId must be passed when the calling the service with complete = true.
    If a valid cadOccId is not supplied when complete = true, the service creates new occurrences and any data
    associated against the old occurrence is lost.
    :var child: Item Revision for the PSOccurrence creation, required reference, if the precise flag is false, then the
    Item will be obtained from the Item Revision and used
    :var occInfo: Member of type RelOccInfo.
    """
    clientId: str = ''
    cadOccId: str = ''
    child: ItemRevision = None
    occInfo: RelOccInfo = None


@dataclass
class RevisionRuleConfigInfo(TcBaseObj):
    """
    This contains the RevisionRule object configuration information.
    
    :var clientId: Identifier that helps the client track the object(s) created
    :var revRule: The RevisionRule object used for configuration of this BOMWindow object.
    :var props: Refers to 'RevisionRuleEntryProps' struct object.
    """
    clientId: str = ''
    revRule: RevisionRule = None
    props: RevisionRuleEntryProps = None


@dataclass
class RevisionRuleEntryProps(TcBaseObj):
    """
    This contains information about the RevisionRule Entry Properties.
    
    :var unitNo: Refers to the unit number of RevisionRule object.
    :var date: Refers to the date of RevisionRule object.
    :var today: Refers to a flag to indicate that the date is today on RevisionRule object.
    :var endItem: Refers to Item and indicates end item for RevisionRule object.
    :var endItemRevision: Refers to ItemRevision and indicates end item revision for RevisionRule object.
    :var overrideFolders: Refers to a list of 'OverrideInfo' struct.
    """
    unitNo: int = 0
    date: datetime = None
    today: bool = False
    endItem: Item = None
    endItemRevision: ItemRevision = None
    overrideFolders: List[OverrideInfo] = ()


@dataclass
class RevisionRuleInfo(TcBaseObj):
    """
    Contains the revisionRule object reference, a map of string attribute to boolean flag to indicate the configurable
    and a list of override information.
    
    :var revRule: RevisionRule object reference
    :var hasValueStatus: A map of string attribute to boolean flag to indicate the configurable attribute has values on
    it and information about override folder.
    :var overrideFolders: List of override information
    """
    revRule: RevisionRule = None
    hasValueStatus: ConfigureAttrStatusMap = None
    overrideFolders: List[OverrideInfo] = ()


@dataclass
class CloseBOMWindowsResponse(TcBaseObj):
    """
    Contains a serviceData containing objects that were deleted.
    
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error information. For this service, ServiceData contains BOM window and top line.
    """
    serviceData: ServiceData = None


@dataclass
class CreateBOMWindowsInfo(TcBaseObj):
    """
    main input structure that defines item or item revision of the top line in the BOM window
    
    :var clientId: Identifier that helps the client track the object(s) created
    :var item: Item object reference for which BOM window needs to create
    :var itemRev: ItemRevision object reference
    :var bomView: PSBOMView object reference
    :var revRuleConfigInfo: Structure with information about the RevisionRuleConfigInfo
    :var objectForConfigure: Tag for Variant rule or option set to use on this BOM window
    :var activeAssemblyArrangement: Active assembly arrangement of this BOM window
    """
    clientId: str = ''
    item: Item = None
    itemRev: ItemRevision = None
    bomView: PSBOMView = None
    revRuleConfigInfo: RevisionRuleConfigInfo = None
    objectForConfigure: BusinessObject = None
    activeAssemblyArrangement: AssemblyArrangement = None


@dataclass
class CreateBOMWindowsOutput(TcBaseObj):
    """
    The output structure that contains the created BOMWindow object and top line BOMLine object representing the item
    or item revision.
    
    :var clientId: Identifier that helps the client track the object(s) created
    :var bomWindow: Object reference for the BOMWindow created
    :var bomLine: Oject reference for the BOMLine created
    """
    clientId: str = ''
    bomWindow: BOMWindow = None
    bomLine: BOMLine = None


@dataclass
class CreateBOMWindowsResponse(TcBaseObj):
    """
    Contains list of CreateBOMWindowsOutput which contains created BOMWindow object and top line BOMLine object
    representing the item or item revision along with the client id and a serviceData containing objects that were
    created/deleted.
    
    :var output: List of CreateBOMWindowsOutput which contains created BOMWindow object and top line BOMLine object
    representing the item or item revision along with the client id
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error information. For this service, ServiceData contains BOM window and top line.
    """
    output: List[CreateBOMWindowsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateAbsoluteStructureInfo(TcBaseObj):
    """
    Contains item revision object reference of the context assembly to create/validate the occurrence, list of
    AbsOccInfo for bvr qualified overrides and a list of AssemblyArrangementInfo for bvr/arrangement qualified
    overrides.
    
    :var contextItemRev: ItemRevision object reference of the context assembly to create/validate the occurrence,
    required reference
    :var bvrAbsOccInfo: List of AbsOccInfo for bvr qualified overrides
    :var arrAbsOccInfo: List of AssemblyArrangementInfo for bvr/arrangement qualified overrides, may be null
    """
    contextItemRev: ItemRevision = None
    bvrAbsOccInfo: List[AbsOccInfo] = ()
    arrAbsOccInfo: List[AssemblyArrangementInfo] = ()


@dataclass
class CreateOrUpdateAbsoluteStructurePref(TcBaseObj):
    """
    Contain cadOccIdAttrName which identifies the BOMLine attribute that is used to identify relative occurrences to
    update.
    
    :var cadOccIdAttrName: Identifies the BOMLine attribute that is used to identify relative occurrences to update.
    """
    cadOccIdAttrName: str = ''


@dataclass
class CreateOrUpdateAbsoluteStructureResponse(TcBaseObj):
    """
    Contains response for createOrUpdateAbsoluteStructure operation.
    
    :var absOccOutput: Map of input clientId for the absolute occurrence to created/updated/found absolute occurrence
    :var asmArrOutput: Map of input client id to created/updated/found AssemblyArrangement
    :var serviceData: The ServiceData contains any other created (AbsOccData, AbsOccDataQualifier), updated (like BVR),
    relevant related, or deleted objects from this operation.
    """
    absOccOutput: ClientIdToAbsOccMap = None
    asmArrOutput: ClientIdToAsmArrMap = None
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateRelativeStructureInfo(TcBaseObj):
    """
    Contains a parent ItemRevision object, list of type RelativeStructureChildInfo and a boolean value for whether the
    BVR should be set to precise.
    
    :var parent: ItemRevision object reference for which the context assembly is created or updated, required reference
    :var childInfo: List of type RelativeStructureChildInfo
    :var precise: Flag for updating the BVR to precise(true)/imprecise(false)
    """
    parent: ItemRevision = None
    childInfo: List[RelativeStructureChildInfo] = ()
    precise: bool = False


@dataclass
class CreateOrUpdateRelativeStructurePref(TcBaseObj):
    """
    Contains cadOccIdAttrName and a list of item types.
    
    :var cadOccIdAttrName: String representing the occurrence note type which holds the value for the CAD occurrence id
    or PSOccurrenceThread uid
    :var itemTypes: List of item types that the client is interested in, such that if the overall structure in
    Teamcenter contains structure relating to other item types or subtypes not in this list, that structure will not be
    deleted if this operation is complete.
    """
    cadOccIdAttrName: str = ''
    itemTypes: List[str] = ()


@dataclass
class CreateOrUpdateRelativeStructureResponse(TcBaseObj):
    """
    The response for createOrUpdateRelativeStructure operation.
    
    :var output: Member of type ClientIdToPSOccurrenceThreadMap
    :var serviceData: Mwmber of type Teamcenter::Soa::Server::ServiceData serviceData
    """
    output: ClientIdToPSOccurrenceThreadMap = None
    serviceData: ServiceData = None


@dataclass
class DeleteAssemblyArrangementsInfo(TcBaseObj):
    """
    Contains ItemRevision object reference and list of AssemblyArrangement.
    
    :var itemRev: ItemRevision object reference
    :var arrangements: List of AssemblyArrangement object references to be deleted
    """
    itemRev: ItemRevision = None
    arrangements: List[AssemblyArrangement] = ()


@dataclass
class DeleteAssemblyArrangementsResponse(TcBaseObj):
    """
    The response for the 'deleteAssemblyArrangements' operation.
    
    :var serviceData: The 'ServiceData'.  This operation will populate the 'ServiceData' with deleted assembly
    arrangements. Assembly arrangement UIDs will be returned as deleted objects.
    """
    serviceData: ServiceData = None


@dataclass
class DeleteRelativeStructureInfo(TcBaseObj):
    """
    Contains parent item revision and list of child information structures.
    
    :var parent: ItemRevision object reference for the context assembly from which children are to be removed
    :var childInfo: List of identifiers of the relative occurrences to be deleted. This is the CAD occurrence id or
    PSOccurrenceThread uid to uniquely identify the occurrence under a particular context Item Revision
    """
    parent: ItemRevision = None
    childInfo: List[str] = ()


@dataclass
class DeleteRelativeStructurePref(TcBaseObj):
    """
    Contains cadOccIdAttrName.
    
    :var cadOccIdAttrName: BOMLine attribute name that contains the CAD occurrence identifier.
    """
    cadOccIdAttrName: str = ''


@dataclass
class DeleteRelativeStructureResponse(TcBaseObj):
    """
    The response for 'deleteRelativeStructure' operation.
    
    :var serviceData: The 'ServiceData'.  This operation will populate the 'ServiceData 'with updated context BVR
    objects and the deleted relative occurrences. CAD occurrence IDs or PSOccurrenceThread UIDs will be returned as
    deleted objects.
    """
    serviceData: ServiceData = None


class BOMLineFilter(Enum):
    """
    BOMLineFilter. 
    Legal values are :  'None, ExcludeOverridden, ExcludeICHistory, ExcludeGDEs, ExcludeNonImanItemLines'.
    """
    None_ = 'None'
    ExcludeOverridden = 'ExcludeOverridden'
    ExcludeICHistory = 'ExcludeICHistory'
    ExcludeGDEs = 'ExcludeGDEs'
    ExcludeNonImanItemLines = 'ExcludeNonImanItemLines'


"""
ItemRevisionToVariantRulesMap
"""
ItemRevisionToVariantRulesMap = Dict[ItemRevision, List[VariantRule]]


"""
ClientIdToAbsOccMap
"""
ClientIdToAbsOccMap = Dict[str, AbsOccurrence]


"""
ClientIdToAsmArrMap
"""
ClientIdToAsmArrMap = Dict[str, AssemblyArrangement]


"""
ClientIdToPSOccurrenceThreadMap
"""
ClientIdToPSOccurrenceThreadMap = Dict[str, PSOccurrenceThread]


"""
ConfigureAttrStatusMap
"""
ConfigureAttrStatusMap = Dict[str, bool]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ReferencedContexts(TcBaseObj):
    """
    the structure used for three different modes: add, set, remove
    
    :var context: the top line of BOP window which will be referenced
    :var addRefContexts: contexts, which will reference
    :var removeRefContexts: contexts, which will be removed
    :var removeExistingRef: remove all referenced contexts
    """
    context: BusinessObject = None
    addRefContexts: List[BusinessObject] = ()
    removeRefContexts: List[BusinessObject] = ()
    removeExistingRef: bool = False


@dataclass
class ReferencedContextsResponse(TcBaseObj):
    """
    the structure contains referenced contexts and used for response.
    
    :var refcontexts: vector of vectors referenced contexts according to the order from the input vector
    :var serviceData: service data will return errors only. No data will be return via Service Data.
    """
    refcontexts: List[ContextsArray] = ()
    serviceData: ServiceData = None


@dataclass
class ContextsArray(TcBaseObj):
    """
    the structure contains array of contexts
    
    :var contextsarray: array of contexts ( any types)
    """
    contextsarray: List[BusinessObject] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateRelationInput(TcBaseObj):
    """
    Input struct for createRelations operation.
    
    :var clientId: Unique client identifier, optional.
    :var relationType: The relation type to create between the two input Business Objects. For example: FND_TraceLink,
    PSConnection, Content. You can get all the supported relation type decriptions by service opertion NetworkEngine
    .getView2().
    :var primaryObject: The primary object to create relation from.
    :var secondaryObject: The secondary object to create relation to.
    :var props: The (string, string) map contains properties for relation creation.
    """
    clientId: str = ''
    relationType: str = ''
    primaryObject: BusinessObject = None
    secondaryObject: BusinessObject = None
    props: StringPropertyMap = None


@dataclass
class CreateRelationsResponse(TcBaseObj):
    """
    Output struct for createRelations operation.
    
    :var clientId: Identifier that helps the client to track the relation(s) created.
    :var edges: The list of new created network edges which representing the relation between Business Objects.
    :var nodeUpdates: The (string,Node) map contains the uid of input node and output node to be updated. This is
    especially useful for BOM struction creation, as the input is uid of ItemRevision, while the output is a BomLine
    node, so client can update network if necessary.
    :var serviceData: Service data including partial errors.
    """
    clientId: str = ''
    edges: List[Edge] = ()
    nodeUpdates: NodeUpdateMap = None
    serviceData: ServiceData = None


@dataclass
class GraphStyleDefResponse(TcBaseObj):
    """
    graph style definition
    
    :var styleXMLStr: XML string contains graph style definition.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors is used
    to report any partial failures.
    """
    styleXMLStr: str = ''
    serviceData: ServiceData = None


@dataclass
class GraphTypeListResponse(TcBaseObj):
    """
    Graph type response.
    
    :var views: A list of  role based avaliable graph views. Different role may get different view list.
    :var serviceData: Service data contains the partial errors which used to report any partial failures.
    """
    views: List[GraphViewDescription] = ()
    serviceData: ServiceData = None


@dataclass
class GraphViewDescription(TcBaseObj):
    """
    The description of a graph view
    
    :var name: Name of the graph view. It is also the identifier of graph view.
    :var defaultLayout: the default layout of the view
    :var defaultExpandDirection: the default expansion direction of the view
    :var visible: Flag to indicate whether the view is visible on graph control panel.
    :var diagramMode: Flag to indicate whether the graph view is in diagram mode by default.
    :var parameters: The list of graph view parameters.
    :var inquiries: The list of supported graph view inquiries. For example: numOfNodes, numOfEdges.
    :var groups: The list of graph view groups.
    """
    name: str = ''
    defaultLayout: str = ''
    defaultExpandDirection: str = ''
    visible: bool = False
    diagramMode: bool = False
    parameters: List[GraphParameterInfo] = ()
    inquiries: List[GraphInquiry] = ()
    groups: List[GraphGroup] = ()


@dataclass
class NetworkResponse(TcBaseObj):
    """
    network response
    
    :var graph: Graph data of the network.
    :var serviceData: Service data contains the list of Teamcenter business objects and also the partial errors is used
    to report any partial failures.
    """
    graph: Graph = None
    serviceData: ServiceData = None


@dataclass
class Node(TcBaseObj):
    """
    node of the graph
    
    :var id: Node id. A unique immutable identifier of the node. It is business object UID for Teamcenter data source.
    :var name: Name of node
    :var metaObject: Underlying Teamcenter object. For example: BomLine, Item, ItemRevision, Function etc. Can be null.
    For external data source, it can be RuntimeObject.
    :var props: A map of property name and property value pairs (string/string). For example, [in_degree, 2],
    [out_degree, 4].
    """
    id: str = ''
    name: str = ''
    metaObject: BusinessObject = None
    props: StringPropertyMap = None


@dataclass
class QueryNetworkInputs(TcBaseObj):
    """
    The input structure containing parameters for query network.
    
    :var rootIds: A list of object IDs that input as network root nodes. If data source is Teamcenter, it is the UID
    list of Teamcenter business objects.
    :var viewName: The name of graph view that will be used for network expansion. Call getViews() service method to
    get available view names.
    :var queryMode: The query mode that applied to this query operation.
    :var serviceCursor: The cursor of the start service index in service list. It's set to 0 for initial call. If
    client get a partial graph by queryNetwork2 operation, it can be set great than 0 to get remaining part of graph.
    :var graphParamMap: A map of graph parameter name and parameter value list (string / list of string), such as
    expansion level, expansion direction. Parameters are inputs leveraged by the graph generation logic. Call
    getViews() service method to get available graph parameters supported by the specified view.
    :var inquiries: A list of inquiries.. Inqueries identify calculations that are performed using the graph contents
    as inputs. The full supported inquires can be got by calling getView() method. Currently supported inquires:
    numOfNodes, numOfEdges.
    """
    rootIds: List[str] = ()
    viewName: str = ''
    queryMode: QueryModeEnum = None
    serviceCursor: int = 0
    graphParamMap: GraphParamMap = None
    inquiries: List[str] = ()


@dataclass
class RGBValue(TcBaseObj):
    """
    Structure for representing color using RGB values, used in structure GraphFilter.
    
    :var redValue: The red component of color.
    :var greenValue: The green component of color.
    :var blueValue: The blue component of color.
    """
    redValue: float = 0.0
    greenValue: float = 0.0
    blueValue: float = 0.0


@dataclass
class Type(TcBaseObj):
    """
    The graph element type structure, used in structure GraphFilter.
    
    :var internalName: The internal type name.
    :var displayName: The display name of type.
    """
    internalName: str = ''
    displayName: str = ''


@dataclass
class Edge(TcBaseObj):
    """
    The structure of edge.
    
    :var leftId: Left node id
    :var rightId: Right node id.
    :var relationType: The logical type of the edge. It is localized.
    :var metaObject: Underlining Teamcenter object.
    :var startPortObject: The start port object associated with the meta object.
    :var endPortObject: The end port object associated with the meta object.
    :var props: A map of property name and property value pairs  (string/string).
    """
    leftId: str = ''
    rightId: str = ''
    relationType: str = ''
    metaObject: BusinessObject = None
    startPortObject: BusinessObject = None
    endPortObject: BusinessObject = None
    props: StringPropertyMap = None


@dataclass
class Graph(TcBaseObj):
    """
    Structure of network graph data.
    
    :var viewName: Graph name of the network.
    :var rootIds: A list of graph root IDs. If data source is Teamcenter, it is the UID list of Teamcenter business
    objects.
    :var serviceCursor: The cursor point to query service that going to be called. For initial query, set service
    cursor to 0.
    :var nodes: A list of Node structures.
    :var edges: A list of Edge structures.
    :var analysisResult: A (string,string) map of inquery and answer pairs.
    :var isPartial: The flag indicates whether it's a partial graph.
    """
    viewName: str = ''
    rootIds: List[str] = ()
    serviceCursor: int = 0
    nodes: List[Node] = ()
    edges: List[Edge] = ()
    analysisResult: GraphInquiryMap = None
    isPartial: bool = False


@dataclass
class GraphFilter(TcBaseObj):
    """
    Structure for representing information of a filter item, including filter name, RGB values to color the legend of
    the filter at client side. It is used in structure GraphGroup.
    
    :var name: name of the filter
    :var color: The color used for legend of this filter at client side
    :var types: The list of types belong to this category.
    """
    name: str = ''
    color: RGBValue = None
    types: List[Type] = ()


@dataclass
class GraphGroup(TcBaseObj):
    """
    The graph group is a group name with a list of filters. A filter take effect either on node or edge, it filts out
    graph that fall into its type. It is used in structure GraphViewDescription.
    
    :var name: Backend name of the group. Currently, there are two available groups, relations and objects.
    :var filters: The list of graph group filter names. For example: the relations graph group may have Attach,
    Tracability, Structure, Folder, Connectivity as filters. The objects graph group may have Requirement, Functional,
    Logical, Physical, Dataset, Plant as filters.
    """
    name: str = ''
    filters: List[GraphFilter] = ()


@dataclass
class GraphInquiry(TcBaseObj):
    """
    The supported inquiry of graph.
    
    :var name: Backend name of the inquiry. A inquery is a short query clause that can be used to retrieve information
    from network graph.
    :var description: Description of the inquiry. Not localized.
    """
    name: str = ''
    description: str = ''


@dataclass
class GraphParameterInfo(TcBaseObj):
    """
    The info of graph parameter
    
    :var name: Backend name of the parameter.
    :var valueMask: The regular expression that used to validate the parameter value.
    :var type: The value type of the parameter. Currently supported types: int, double, string.
    :var defaultValue: Default value of the parameter
    :var description: the description of the parameter. Not localized.
    """
    name: str = ''
    valueMask: str = ''
    type: str = ''
    defaultValue: str = ''
    description: str = ''


class QueryModeEnum(Enum):
    """
    The query mode enumeration.
    
    :var DegreeOnly: Only calculate node degree without return graph.
    :var ExpandOnly: Only expand graph without calculate node degree.
    :var ExpandAndDegree: Expand graph and calculate node degree together.
    """
    DegreeOnly = 'DegreeOnly'
    ExpandOnly = 'ExpandOnly'
    ExpandAndDegree = 'ExpandAndDegree'


"""
the inquiry/answer map
"""
GraphInquiryMap = Dict[str, str]


"""
Network graph parameter map
"""
GraphParamMap = Dict[str, List[str]]


"""
The map with the key of input node uid to output node.
"""
NodeUpdateMap = Dict[str, Node]


"""
The map contains key and value for string property.
"""
StringPropertyMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class FindMatchingCandidatesResp(TcBaseObj):
    """
    FindMatchingCandidatesResp structures contains a list of MatchingCandidateElems capturing the matching pairs of
    source BOMLine objects and target BOMLine objects, and serviceData to return partial errors. The size of the list
    of MatchingCandidateElems will be the same as the size of input list to findMatchingCandidates operation.
    
    :var matchedCandidates: A list of MatchingCandidateElems element.
    :var serviceData: Service data capturing partial errors.
    """
    matchedCandidates: List[MatchingCandidateElems] = ()
    serviceData: ServiceData = None


@dataclass
class MatchingCandidateElemForSingleObj(TcBaseObj):
    """
    A structure representing response for each BOMLine object in the source BOMLine objects in input.
    
    :var objectToSearchFor: The BOMLine object for which equivalent target BOMLine objects were requested.
    :var matchingCandidates: The list of target objects matching the source object.
    :var howMatched: A string providing details of how the target match was found. This is for future and will be empty
    in current implementation.
    """
    objectToSearchFor: BusinessObject = None
    matchingCandidates: List[BusinessObject] = ()
    howMatched: List[str] = ()


@dataclass
class MatchingCandidateElems(TcBaseObj):
    """
    A list of MatchingCandidateElemForSingleObj elements used to group them per input element - MatchingCandidateElem
    
    :var matches: A list of MatchingCandidateElemForSingleObj elements.
    """
    matches: List[MatchingCandidateElemForSingleObj] = ()


@dataclass
class SearchCandidateElems(TcBaseObj):
    """
    A structure to capture the details of searching for matched targetlines for given source lines. If both
    searchCandidates and scopesToSearchIn are specified, the searchCandidates will take priority. Typically, only one
    of either searchCandidates or scopesToSearchIn will be specified.
    
    :var objectsToSearchFor: The list of source objects for which target candidates need to be found.
    :var searchCandidates: The list of target objects from which the matching candidates are to be found.
    :var scopesToSearchIn: The list of target scope elements to be used for finding matching candidates.
    """
    objectsToSearchFor: List[BusinessObject] = ()
    searchCandidates: List[BusinessObject] = ()
    scopesToSearchIn: List[TargetScopeElem] = ()


@dataclass
class TargetScopeElem(TcBaseObj):
    """
    A structure representing the scopes and the closure rule to collect possible candidates for alignment with source
    lines. If the closure rule is empty, it is assumed that all children under the scopes are to be used for finding
    candidates.
    
    :var scopes: A list of objects to be used as scopes for searching for suitable candidate lines.
    :var closureRule: Name of closure rule to be used for collecting all possible below the scopes.
    """
    scopes: List[BusinessObject] = ()
    closureRule: str = ''


@dataclass
class AdditionalData(TcBaseObj):
    """
    A structure to capture addition information in a generic manner.
    
    :var intMap: map of string to vector or integers.
    :var dblMap: map of string to vector of doubles.
    :var strMap: map of string to vector of strings.
    :var objMap: map of string to vector of objects.
    :var dateMap: map of string to vector of dates
    """
    intMap: StringToIntVectorMap = None
    dblMap: StringToDblVectorMap = None
    strMap: StringToStrVectorMap = None
    objMap: StringToObjVectorMap = None
    dateMap: StringToDateVectorMap = None


@dataclass
class AlignMatchedCandidateElem(TcBaseObj):
    """
    A structure to capture the information necessary to align matching target BOMLine objects.
    
    :var sourceObject: The Engineering BOMLine object.
    :var targetObject: The Manufacturing BOMLine object to be synced or replaced by the Engineering BOMLine objects'
    item.
    :var alignAction: The specific action to take for alignment. Possible values are: empty  "" -just align the two
    BOMLine objects ( sync bl_abs_occ_id property ), "replace" - the manufacturing BOMLine object is to be replaced
    with the Engineering BOMLine object. "propagate" - propagate the property values from sourceObject to targetObject.
    Any other custom values are passed to custom handlers registered against user_exit - USER_BOMLine_repair_bomlines.
    """
    sourceObject: BusinessObject = None
    targetObject: BusinessObject = None
    alignAction: str = ''


@dataclass
class AlignMatchedCandidatesResp(TcBaseObj):
    """
    AlignMatchedCandidatesResp structure contains a generic additionalInfo structure and serviceData to return partial
    errors. Currently, the additionalInfo is unused.
    
    :var additionalInfo: A generic structure of type AdditionalData containing keys and list of values (integer,
    double, object, string, date types) for passing additional metadata. Currently this is not used.
    :var serviceData: Service data capturing partial errors.
    """
    additionalInfo: AdditionalData = None
    serviceData: ServiceData = None


"""
a map of string to vector of dates
"""
StringToDateVectorMap = Dict[str, List[datetime]]


"""
String to vector of doubles map.
"""
StringToDblVectorMap = Dict[str, List[float]]


"""
map of string to vector of integers.
"""
StringToIntVectorMap = Dict[str, List[int]]


"""
a map of string to vector of objects.
"""
StringToObjVectorMap = Dict[str, List[BusinessObject]]


"""
a map of String to vector of strings.
"""
StringToStrVectorMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.AWS2._2018_12.RequirementsManagement import CreateTracelinksInputData, CreateTracelinksResponse
from typing import List
from tcsoa.utils import TcService


class RequirementsManagementService(TcService):

    @classmethod
    def createTracelinks(cls, input: List[CreateTracelinksInputData]) -> CreateTracelinksResponse:
        """
        This operation creates a FND_TraceLink object between the primary and secondary business objects.
        
        Use cases:
        In ACE perspective, user select an Awb0Element element and clicks on the "Create TraceLink" button. The
        selected element will be in the Create Trace Link panel start section, user chooses another Awb0Element element
        in the Create Trace Link panel end section. 
        
        - If createTracelinkWithOccurrences mode is true, the trace link between the Occurrences will be created. 
        - If createDefiningTracelinkWithOccurrence is true, then the trace link between the defining Occurrence and the
        complying underlying workspace object will be created.
        - If createComplyingTracelinkWithOccurrence is true, then the trace link between the defining underlying
        workspace object and the complying Occurrence will be created.
        
        """
        return cls.execute_soa_method(
            method_name='createTracelinks',
            library='AWS2',
            service_date='2018_12',
            service_name='RequirementsManagement',
            params={'input': input},
            response_cls=CreateTracelinksResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class OkToCheckoutResponse(TcBaseObj):
    """
    This structure contains the list and 'ServiceData' objects. The list contains verdict true or false for passed in
    object indicating if object can be checked out or not. 'ServiceData' contains error at index where it occurred for
    each input object.
    
    :var verdict: A list indicating if the input business object can be checked out. "true" indicates the object may be
    checked out. "false" indicates the object may not be checked out.
    :var serviceData: Contains the partial errors for any objects for which the 'okToCheckout' validation failed.
    """
    verdict: List[bool] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Cfg0ConfiguratorPerspective
from tcsoa.gen.Internal.Configurator._2015_10.ConfiguratorManagement import BusinessObjectConfigExpression
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ConfigurationProfile(TcBaseObj):
    """
    Structure consisting maps of different primitive types.
    Following are the Configuration Profile keys that can be passed:
    - applyConfigConstraints  - Boolean - Enables configurator constraint evaluation in the solve operation.
    - applyDefaults - Boolean - Enables configurator default rules evaluation in the solve operation.
    - intentFilterInclude - List of Strings - The list of configurator Intents which should participate in the solve
    operation.
    - intentFilterExclude - List of Strings - The list of configurator Intents which should be ignored in the solve
    operation.
    - violationComputationTimeout - Integer - Indicates the max time in seconds that should be spent in violations
    computation.
    - minReportSeverity - Integer - Indicates the minimum report severity to be evaluated for validation failures.
    - minErrorSeverity - Integer - Indicates the minimum error severity to be evaluated for validation failures.
    - computeAllProblems - Boolean - Enables computation of all the problems of the unsat core while reporting
    violations.
    - minimizeProblem - Boolean - Decides if minimization of computed essential violation is required.
    
    
    
    :var intMap: Map (string/list of ints) of configuration Profile preferences of Integer type.
    :var boolMap: Map (string/list of booleans) of configuration Profile preferences of Boolean type.
    :var stringMap: Map (string/list of strings) of configuration Profile preferences of String type.
    :var doubleMap: Map (string/list of doubles) of configuration Profile preferences of Double type.
    :var objectMap: Map (string/list of business objects) of configuration Profile preferences of Business Object type.
    :var dateMap: Map (string/list of dates) of configuration Profile preferences of Date type.
    """
    intMap: IntVectorMap2 = None
    boolMap: BoolVectorMap2 = None
    stringMap: StringVectorMap2 = None
    doubleMap: DoubleVectorMap2 = None
    objectMap: ObjectVectorMap2 = None
    dateMap: DateVectorMap2 = None


@dataclass
class ConfigurationSessionInfoInput(TcBaseObj):
    """
    An input structure containing list of configurator perspective object, configuration profile map and a vector of
    variant rule objects.
    
    :var perspective: The Cfg0ConfiguratorPerspective instance which provides the RevisionRule and the Configurator
    Context.
    :var configProfile: The Configuration Profile information required for Configurator Solver.
    :var targetObjects: A list of target objects on which the Configuration Profile information needs to be set.
    """
    perspective: Cfg0ConfiguratorPerspective = None
    configProfile: ConfigurationProfile = None
    targetObjects: List[BusinessObject] = ()


@dataclass
class GetContextBasedVariantExprsResponse(TcBaseObj):
    """
    The response structure for operation getContextBasedVariantExpressions.
    
    :var configObjectExpressions: A list of structures each relating a business object to a set of configuration
    expressions.
    :var serviceData: Teamcenter service data.
    """
    configObjectExpressions: List[BusinessObjectConfigExpression] = ()
    serviceData: ServiceData = None


"""
Map of String to List of Integer values.
"""
IntVectorMap2 = Dict[str, List[int]]


"""
Map of String to List of BO values.
"""
ObjectVectorMap2 = Dict[str, List[BusinessObject]]


"""
Map of String to List of Boolean values.
"""
BoolVectorMap2 = Dict[str, List[bool]]


"""
Map of String to List of String values.
"""
StringVectorMap2 = Dict[str, List[str]]


"""
Map of String to List of DateTime values.
"""
DateVectorMap2 = Dict[str, List[datetime]]


"""
Map of String to List of Double values.
"""
DoubleVectorMap2 = Dict[str, List[float]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateAndPasteInputInfo(TcBaseObj):
    """
    CreateAndPasteInputInfo structure represents the information necessary to create a new object and paste the same.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data     elements and
    partial errors associated     with this input structure.
    :var opType: createAndPaste - use in cases where create     and paste both are needed
    pasteOnly    - use in cases where paste alone     is needed.newObject should have valid value     in this case.
    
    :var createData: Input data for create operation.
    :var newObject: The object to paste in pasteTarget in case of pasteOnly else it is NULL.
    
    :var pasteTarget: The target object onto which the newly created object is to be pasted.
    :var pasteRelation: The relation to create between the object     being pasted and paste target.If no value is    
    specified the default relation is used.
    """
    clientId: str = ''
    opType: str = ''
    createData: CreateInput = None
    newObject: BusinessObject = None
    pasteTarget: BusinessObject = None
    pasteRelation: str = ''


@dataclass
class CreateAndPasteOutput(TcBaseObj):
    """
    This structure represents the details of the output of CreateAndPaste operation.
    
    :var clientID: The clientId from the input CreateAndPasteInputInfo element.This value is unchanged from the input,
    and can be used to identify this response elementwith the corresponding input element.
    :var pasteResultObject: Runtime object resulting from the paste of newly created object.This can be null in some
    cases.E.g: Paste in a Folder object.
    :var newObject: Newly created object.
    """
    clientID: str = ''
    pasteResultObject: BusinessObject = None
    newObject: BusinessObject = None


@dataclass
class CreateAndPasteResponse(TcBaseObj):
    """
    The structure represents the output of CreateAndPaste operation.It contains the CreateAndPasteOutput object and
    ServiceData.
    
    :var output: CreateAndPasteOutput objects containing information about the newly created object and paste result
    object.
    :var serviceData: The ServiceData.
    """
    output: List[CreateAndPasteOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateConnectionPortsAndConnectInputInfo(TcBaseObj):
    """
    This structure contains the details required for creating, pasting port and connection objects and establishing
    connection between them.
    
    :var clientId: A unique string supplied by the caller. This ID is used  to identify return data elements and
    partial errors associated with this input structure.
    :var inputMap: A map of key input data (string/CreateAndPasteInputInfo) valid keys are PORT1,PORT2,CONNECTION
    :var connectCase: It can be one of the following values 
    - create2PortsConxAndConnToPorts -  Use this to create    two ports, connection and establish connection between
    them.
    - create1PortConxAndConnToPorts - Uset this to create one    port, connection and establish connection between the 
      existing port,newly created port and connection.
    - createConxConnectToPorts - Use this to create a connection and connect it to existing ports.
    - create2PortsConxExistsConnToPorts- Use this to create two ports and connect the existing connection to these
    ports.
    - create1PortConxExistsConnToPorts - Use this to create a    single port and then connect the existing connection
    to this port and existing port.
    - createConxConnToBlocks- Use this to establish connection between blocks [preference connected_ToRules should also
    allow this]
    
    
    :var objectsToConnectTo: Ports / Blocks to connect the connection.
    :var connectionObject: The Connection object if already exists.
    """
    clientId: str = ''
    inputMap: ConnectInputDataMap = None
    connectCase: str = ''
    objectsToConnectTo: List[BusinessObject] = ()
    connectionObject: BusinessObject = None


@dataclass
class CreateConnectionPortsAndConnectResponse(TcBaseObj):
    """
    This structure represents the output of createConnectionPortsAndConnect operation. It contains ConnectOutputDataMap
    object[s] and ServiceData.
    
    :var output: A map of key output data (string/ CreateAndPasteOutput) .Valid keys are PORT1,PORT2,CONNECTION
    :var serviceData: ServiceData
    """
    output: List[ConnectOutputDataMap] = ()
    serviceData: ServiceData = None


@dataclass
class CreateInput(TcBaseObj):
    """
    The parameters required to create the Business Object.
    
    :var boName: Business Object type name
    :var propertyNameValues: Map of property name (key) and property values (values) in string format, to be set on new
    object being created. Note: The calling client is responsible for converting the different property types (int,
    float, date .etc) to a string using the appropriate function(s) in the SOA client framework Property class.
    :var compoundCreateInput: CreateInput for compounded objects.
    """
    boName: str = ''
    propertyNameValues: PropNameValueMap = None
    compoundCreateInput: CreateInputMap = None


"""
Map of reference or relation property name to secondary CreateInput objects.
"""
CreateInputMap = Dict[str, List[CreateInput]]


"""
This map contains the property name and its values.
"""
PropNameValueMap = Dict[str, List[str]]


"""
Map of reference or relation property name to CreateAndPasteInputInfo     objects.
"""
ConnectInputDataMap = Dict[str, CreateAndPasteInputInfo]


"""
Map of reference or relation property name to secondary CreateAndPasteOutput    objects
"""
ConnectOutputDataMap = Dict[str, CreateAndPasteOutput]



from __future__ import annotations

from tcsoa.gen.Workflow._2015_07.Workflow import CreateSignoffs
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class WorkflowService(TcService):

    @classmethod
    def addSignoffs(cls, signoffs: List[CreateSignoffs]) -> ServiceData:
        """
        This operation adds signoffs to a workflow task. The signoffs can be added as adhoc-signoffs or it can be used
        to staff profiles. The signoff members can be group members or resource pools. When an address list is
        provided, the members of the address list are used to create and add signoffs to the task. The signoffs can be
        added as review signoffs, acknowledge signoffs and notify signoffs. This operation allows to designate added
        reviewers as required or optional.
        
        Use cases:
        Following are four different use cases to add the signoffs &ndash;
        Use case 1: Add adhoc signoff
        To add an adhoc signoff, signoffMember specified as part of 'CreateSignoffInfo' structure can either be a
        GroupMember, Resource Pool or an address list. For this use case, value for 'origin' in 'CreateSignoffInfo'
        should be NULLTAG.
        
        Use case 2: Add profile signoff
        In this use case, the signoffMember specified as part of the 'CreateSignoffInfo' structure has to satisfy the
        signoff profile. Value for the 'origin' should be the signoff profile object and 'originType' should be
        "SOA_EPM_SIGNOFF_ORIGIN_PROFILE".
        
        Use case 3: Add subset of the  members of the  address list as signoff
        User may want to add subset of the members of an address list as signoffs. In this use case, the 'origin'
        should be the address list object and 'originType' should be "SOA_EPM_SIGNOFF_ORIGIN_ADDRESSLIST".  Note that
        members of the address list are always added as adhoc signoffs.
        
        Use case 4: Add signoff as required or optional
        User may specify the signoff decision is required or optinal for the added reviewer . By default the signoff is
        "Optional" signoff.This can be done by adding string member 'signoffRequired' in 'CreateSignoffInfo'.To
        designate the added reviwer as required signoff the string value will be "RequiredUnmodifiable" which cannot be
        manually overridden to "Optional". One Possible value is "RequiredModifiable" It indicates sign off decision is
        required, which can be manually overridden to "Optional".
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException:
        """
        return cls.execute_soa_method(
            method_name='addSignoffs',
            library='Workflow',
            service_date='2015_07',
            service_name='Workflow',
            params={'signoffs': signoffs},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Ai._2016_04.Ai import InvokeResponse
from tcsoa.utils import TcService


class AiService(TcService):

    @classmethod
    def invoke2(cls, methodName: str, xmlIn: str) -> InvokeResponse:
        """
        This operation is for backward compatibility with existing NX clients that use this method to call custom PDI
        functions. This method will return InvokeResponse structure containing plain objects that are also returned as
        part of formatted XML. These plain objects can be used by test framework for tracking purposes.
        
        Exceptions:
        >Thrown on service failure. The exception can result from calls to more than 400 PDI APIs and depends on case
        to case basis.
        """
        return cls.execute_soa_method(
            method_name='invoke2',
            library='Internal-Ai',
            service_date='2016_04',
            service_name='Ai',
            params={'methodName': methodName, 'xmlIn': xmlIn},
            response_cls=InvokeResponse,
        )



from __future__ import annotations

from tcsoa.gen.ImportExport._2007_06.FileImportExport import ImportFromApplicationInputData, ExportToApplicationResponse, ImportFromApplicationResponse, ExportToApplicationInputData
from typing import List
from tcsoa.utils import TcService


class FileImportExportService(TcService):

    @classmethod
    def importFromApplication(cls, inputs: List[ImportFromApplicationInputData]) -> ImportFromApplicationResponse:
        """
        This operation imports the contents of the given MSWord document to create objects of type SpecElement.  The
        MSWord document to be imported to Teamcenter should have .docx file format. If the application format is
        MSWordXML then the operation parses the given MSWord document to creates a structure of chosen subtype of
        SpecElement.  The parsing of dcoument involves parsing of individual paragraphs in the document. Each paragraph
        in the document with a Heading becomes a SpecElement. After the BOM structure is created at the server, a
        BOMWIndow is constructed at the server and returned returned to the caller. Objects of type
        Requirement/Paragraph are created at the server after importing the document.  This operation supports Word
        documents in MSOffice Open XML format (.docx format) only. If the application format is not set then the
        operation does not do anything.
        
        Use cases:
        User can create an MSWord document and import it using this operation to create a structure. Each paragraph in
        MSWord document represents a Requirement/Paragraph in the structure. When this operation is called from
        Teamcenter rich client, a structure is created and is opened in the Requirements Management application.  The
        structure and indentation of the child Requirements is driven by the MSWord outline level in the document. 
        Each Paragraph that is formatted in an outline level style produces a separate Requirement. This file may be
        located in a local file system folder or a network folder.
        
        Exceptions:
        >Service Exception    This occurs due to an error during generating transient file ticket due to a
        configuration issue at the server. For example, if the transient volume directory is not configured at the
        server then the FMS fails to import the document and subsequent file upload operation fails. In such situation
        a 'ServiceException' is thrown.
        """
        return cls.execute_soa_method(
            method_name='importFromApplication',
            library='ImportExport',
            service_date='2007_06',
            service_name='FileImportExport',
            params={'inputs': inputs},
            response_cls=ImportFromApplicationResponse,
        )

    @classmethod
    def exportToApplication(cls, inputs: List[ExportToApplicationInputData]) -> ExportToApplicationResponse:
        """
        This operation is used for exporting Teamcenter objects (WorkspaceObject) to applications like MSWord and
        MSExcel. Teamcenter business objects should already be created so that the objects can be exported to Word and
        Excel using the service operation.
        An MSExcel (.mhtml) or a MSWord (.docx) file is generated at the server as a result of this operation.  The
        generated file will contain Teamcenter objects and their properties. Depending upon the application format that
        is being passed as input parameter, the generated file can be opened in Word or Excel.  If the application
        format is "'MSWordXML'" then the generated document is a Word document.  If the application format is
        "'MSExcel'" then the generated sheet is a static Excel spreadsheet.  If the application format is
        "'MSExcelLive'" then the generated sheet is a Live Excel spreadsheet. "Live" sheet means that modifications can
        be done to the data in Excel which will reflect to Teamcenter.
        
        
        
        Use cases:
        User can create Teamcenter objects in RAC and then export those objects and their properties to Word or Excel.
        If "Live" option is selected then User can create "Live" spreadsheet after export to Excel.
        
        Following usecases are supported in this operation
        
        - Export of Workspace objects to MSWord (static)
        - Export of Teamcenter objects to MSExcel(static)
        - Export of Teamcenter objects to MSExcel(Live) and edit the properties from Excel Live sheet.
        
        
        
        Exceptions:
        >Service Exception If there is any error during generating transient file read ticket due to a configuration
        issue at the server, then the operation throws a service exception. Example- If the transient volume directory
        is not configured at the server then the FMS fails to generate a file at the server and subsequent file
        download operation fails. In such situation a service exception is thrown.
        """
        return cls.execute_soa_method(
            method_name='exportToApplication',
            library='ImportExport',
            service_date='2007_06',
            service_name='FileImportExport',
            params={'inputs': inputs},
            response_cls=ExportToApplicationResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetICOMappingTargetsResponse(TcBaseObj):
    """
    -
    
    :var serviceData: The service data containing partial errors if any.
    :var mappingInfoVector: A list of "MappingInfo" structure.
    """
    serviceData: ServiceData = None
    mappingInfoVector: List[MappingInfo] = ()


@dataclass
class MappingInfo(TcBaseObj):
    """
    Structure containing data for  GTC mapping.
    
    :var sourceIcoID: ICO ID for the source class.
    :var sourceIcoUID: ICO UID for the source class.
    :var sourceClassID:  ID for the source class.
    :var sourceClassName: Name of the source class.
    :var targetMachine: Value of machine side connection code used for finding match based on connection codes.
    :var targetWorkpiece: Value of workpiece side connection code used for finding match based on connection codes.
    :var targetGTCBase:  GTC Base value for target class used for finding match based on connection codes.
    :var targetClasses: A list of "TargetClass" structure.
    """
    sourceIcoID: str = ''
    sourceIcoUID: str = ''
    sourceClassID: str = ''
    sourceClassName: str = ''
    targetMachine: str = ''
    targetWorkpiece: str = ''
    targetGTCBase: str = ''
    targetClasses: List[TargetClass] = ()


@dataclass
class TargetClass(TcBaseObj):
    """
    -
    
    :var targetID: ID of target class.
    :var targetName: Name of target class.
    """
    targetID: str = ''
    targetName: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanQuery
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class QueryInput(TcBaseObj):
    """
    The information about each Saved Query to be processed is provided
     by way of this data structure.
    
    :var query: The saved query object to be executed.
    :var entries: The names of criteria. They are User Entry Name of the' query' seen in Query Builder, not User Entry
    L10N Key. To get User Entry Name in different local, use com.teamcenter.rac.kernel.TcTextService.getTextValue() for
    Teamcenter rich client customization, use com.teamcenter.services.strong.core._2008_06.Session.getDisplayStrings()
    for Teamcenter service customization.
    
    :var values: The values of criteria.
    :var limitList: A list of objects (optional) against which the search is conducted,
    if the list is empty, search will be conducted against the database.
    :var maxNumToReturn: A specified maximum number of matches to be returned,
    0 means no limit.
    :var resultsType: The type of results expected from this operation:
    0 (top-level objects only),
    1 (top-level objects plus children: Hierarchical/Indented results),
    2 (default value as specified on the query object).
    :var requestId: Unique ID used to register the query execution task. This can be used by the caller to cancel the
    time consuming query, the value can be generated by any unique string generator.
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    """
    query: ImanQuery = None
    entries: List[str] = ()
    values: List[str] = ()
    limitList: List[BusinessObject] = ()
    maxNumToReturn: int = 0
    resultsType: int = 0
    requestId: str = ''
    clientId: str = ''



from tcsoa.gen.Internal_ProjectManagement._2014_10.services import ScheduleManagementService as imp0
from tcsoa.gen.Internal_ProjectManagement._2008_06.services import ScheduleManagementService as imp1
from tcsoa.gen.Internal_ProjectManagement._2007_06.services import ScheduleManagementService as imp2
from tcsoa.gen.Internal_ProjectManagement._2011_06.services import ScheduleManagementService as imp3
from tcsoa.gen.Internal_ProjectManagement._2007_01.services import ScheduleManagementService as imp4
from tcsoa.gen.Internal_ProjectManagement._2012_02.services import ScheduleManagementService as imp5
from tcsoa.gen.Internal_ProjectManagement._2010_04.services import ScheduleManagementService as imp6
from tcsoa.gen.Internal_ProjectManagement._2009_10.services import ScheduleManagementService as imp7
from tcsoa.utils import TcService


class ScheduleManagementService(TcService):
    deferredSave = imp0.deferredSave
    getSchedulesToinsert = imp1.getSchedulesToinsert
    loadProgramView = imp2.loadProgramView
    loadProgramView2 = imp3.loadProgramView
    loadResourceAssignments = imp2.loadResourceAssignments
    loadSchedule = imp4.loadSchedule
    loadSchedules = imp1.loadSchedules
    modifySchedule = imp4.modifySchedule
    modifySchedule2 = imp2.modifySchedule
    modifySchedule3 = imp1.modifySchedule
    modifySchedules = imp3.modifySchedules
    pasteTasks = imp0.pasteTasks
    translateFive = imp5.translateFive
    translateFour = imp6.translateFour
    translateOne = imp7.translateOne
    translateThree = imp7.translateThree
    translateTwo = imp7.translateTwo



from __future__ import annotations

from tcsoa.gen.Core._2017_11.LogicalObject import GetLogicalObjectResponse, GetLogicalObjectInput
from typing import List
from tcsoa.utils import TcService


class LogicalObjectService(TcService):

    @classmethod
    def getLogicalObjects(cls, loInputs: List[GetLogicalObjectInput]) -> GetLogicalObjectResponse:
        """
        This operation returns the logical object instances for the input list of root object instances and logical
        object type names. This operation can also return classification objects [ A classification object is also
        called an ICO], if the root obejct or a member object is configured on the logical object type for retieving
        classification data. For such usecases it also returns ICO property data in an ICO specific property structure.
        
        This operation returns all properties for given logical object type when property policy is not found.
        
        This operation will have a one-to-one mapping between the input vector and output vector.  The order in the
        input vector is also matched to the order in the output vector. In an error scenario, an empty entry in the
        output vector  will be returned.
        
        Use cases:
        This operation is invoked to retrieve logical object instances by the client by passing a list of root object
        instances and logical object type names.
        
        For example, consider the below use case data for the use case(s) specified:
        
        Use Case Data:
        
        1.  Logical object definition 1
        Name                           :   "Logical Object 1"
        Root business object           :   Item
        
        2.  Logical object definition 2
        Name                           :   "Logical Object 2"
        Root business object           :   ItemRevision
        
        3.  Logical object definition 3
        Name                           :   "Logical Object 3"
        Root business object           :   Item
        
        4.  Logical object definition 4
        Name                           :   "Logical Object 4"
        Root business object           :    ItemRevision
        Retrieve classification data   :    True
        
        Note: The presented property fnd0Root_ICO is automatically generated when "Retrieve classification data" is
        checked for the root business object. The same is applied for a member.
        
        
        Use Case 1:
        
        GetLogicalObjectInput[0] = {
        rootObjects   =   [Item] ,
        loTypeName    =   "Logical Object 1"
         }
        
        then GetLogicalObjectResponse structure would contain loOutputs vector of size = 1
        and element = 1 contains 1 instance(s) of "Logical Object 1"
        
        Use Case 2:
        
        GetLogicalObjectInput[0] = {
        rootObjects   =   [Item] ,
        loTypeName    =   "Logical Object 2"
         }
        
        then GetLogicalObjectResponse structure would contain loOutputs vector of size = 1
        and element = 1 contains 0 instance(s)
        
        
        Use Case 3:
        
        GetLogicalObjectInput[0] = {
        rootObjects   =   [Item, Item] ,
        loTypeName    =   "Logical Object 1"
         }
        
        then GetLogicalObjectResponse structure would contain loOutputs vector of size = 1
        and element = 1 contains 2 instance(s) of "Logical Object 1" and "Logical Object 1"
        
        Use Case 4:
        
        GetLogicalObjectInput[0] = {
        rootObjects   =   [Item, ItemRevision] ,
        loTypeName    =    "Logical Object 1"
         }
        
        then GetLogicalObjectResponse structure would contain loOutputs vector of size = 1
        and element = 1 contains 1 instance(s) of "Logical Object 1"
        
        
        Use Case 5:
        
        GetLogicalObjectInput [0] = {
        rootObjects   =   [Item] ,
        loTypeName    =   "Logical Object 1"
        }
        GetLogicalObjectInput [1] = {
        rootObjects   =   [ItemRevision] ,
        loTypeName    =   "Logical Object 2"
        }
        
        then GetLogicalObjectResponse structure would contain loOutputs vector of size = 2
        and element = 1 returns 1 instance(s) of "Logical Object 1"
        and element = 2 returns 1 instance(s) of "Logical Object 2"
        
        
        Use Case 6:
        
        GetLogicalObjectInput [0] = {
        rootObjects   =   [Item] ,
        loTypeName    =   "Logical Object 1"
        }
        GetLogicalObjectInput [1] = {
        rootObjects   =   [Item] ,
        loTypeName    =   "Logical Object 2"
        }
        GetLogicalObjectInput [2] = {
        rootObjects   =   [Item] ,
        loTypeName    =   "Logical Object 3"
        }
        
        then GetLogicalObjectResponse structure would contain loOutputs vector of size = 3
        and element = 1 returns 1 instance(s) of "Logical Object 1"
        and element = 2 contains 0 instance(s)
        and element = 3 contains 1 instance(s) of "Logical Object 3"
        
        
        Use Case 7:
        GetLogicalObjectInput [0] = {
        rootObjects         =   [ItemRevision] ,
        loTypeName          =   "Logical Object 4"
         }
        
        then GetLogicalObjectResponse structure would contain loOutputs vector of size = 1
        and element = 1 returns 1 instance(s) of "Logical Object 4"and
        GetLogicalObjectOutput structure would contain ClassificationObjectInfoMap map and provide details for the ICO
        objects assocaiated with ItemRevison
        and ClassificationAttributeDescMap map
        would provide details for the classification class of the ICO objects assocaiated with ItemRevison.
        """
        return cls.execute_soa_method(
            method_name='getLogicalObjects',
            library='Core',
            service_date='2017_11',
            service_name='LogicalObject',
            params={'loInputs': loInputs},
            response_cls=GetLogicalObjectResponse,
        )



from __future__ import annotations

from tcsoa.gen.DocumentManagement._2007_01.DocumentTemplate import GetTemplateInput, GetTemplatesResponse
from tcsoa.utils import TcService


class DocumentTemplateService(TcService):

    @classmethod
    def getTemplates(cls, input: GetTemplateInput) -> GetTemplatesResponse:
        """
        This operation is intended to support applications with a Microsoft Office style template system, where the
        user chooses to create a new document, and the application presents the user with a selection of different
        templates to use to create the document. The calling application inputs its search criteria, and getTemplates
        returns the list of DMTemplate objects that match the criteria. The DMTemplate is a business object that is
        used to control the template files provided during Create.
        
        The search criteria are:
        - The name of the calling application. DMTemplate objects are application specific.
        - The version number of the calling application. Optional.
        - The desired measurement type; e.g. inches or metric. Used for CAD applications.
        - The type of Item that will be created using the DMTemplate file.
        - An application defined type value.
        
        
        
        The operation returns a list of DMTemplate names and thumbnail images that can be used by the application to
        present a choice to the user. Once the user selects a DMTemplate to use in the creation of the new object, the
        application can get the files for the selected DMTemplate using the expandPrimaryObjects operation.
        
        Use cases:
        Get a list of templates for creating a new object.
        
        The user decides to create a new ItemRevision in a CAD application. The application calls this operation and
        receives a list of template names and thumbnails to display to the user. The user selects a template and the
        application retrieves the template files using the expandPrimaryObjects operation. The application then creates
        the new ItemRevision and attaches copies of the template files.
        """
        return cls.execute_soa_method(
            method_name='getTemplates',
            library='DocumentManagement',
            service_date='2007_01',
            service_name='DocumentTemplate',
            params={'input': input},
            response_cls=GetTemplatesResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Core._2012_09.DataManagement import RelateInfoIn
from tcsoa.gen.Internal.Core._2016_10.DataManagement import SaveAsIn, ReviseIn
from tcsoa.gen.Core._2013_05.DataManagement import ReviseObjectsResponse
from tcsoa.gen.Core._2011_06.DataManagement import SaveAsObjectsResponse
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def reviseObjectsInBulk(cls, reviseData: List[ReviseIn]) -> ReviseObjectsResponse:
        """
        This operation revises the given objects and copies or creates new objects in bulk using input property values
        and deep copy data. Deep copy processing is recursive such that relations between secondary objects, or from
        secondary objects to the revised object, are replicated during the operation based on deep copy rule
        configurations. This operation supports codeless configuration of custom properties. The following list of
        revisable types are supported for this operation:
        - Identifier and its sub-types
        - ItemRevision and its sub-types
        - Mdl0BaselineRevision and its sub-types
        - Mdl0ConditionalElement and its sub-types
        - Cpd0DesignControlElement and its sub-types
        - Ptn0Partition and its sub-types
        - Cpd0DesignModelElement and its sub-types
        - Cpd0ShapeDesignRevision and its sub-types
        - Bom0ConfigurableBomElement and its sub-types
        - Cfg0AbsConfiguratorWSO and its sub-types
        
        """
        return cls.execute_soa_method(
            method_name='reviseObjectsInBulk',
            library='Internal-Core',
            service_date='2016_10',
            service_name='DataManagement',
            params={'reviseData': reviseData},
            response_cls=ReviseObjectsResponse,
        )

    @classmethod
    def saveAsObjectsInBulkAndRelate(cls, saveAsData: List[SaveAsIn], relationInfo: List[RelateInfoIn]) -> SaveAsObjectsResponse:
        """
        This operation performs SaveAs operation in bulk on the input target business objects and its related objects
        as new instances. Related objects are identified using deep copy rules. Optionally, this operation relates the
        new object to the input target object or to a default folder and in this case the size of saveAsData and the
        size of relationInfo have to be the same. The following list of types are supported for this operation: 
        
        - ItemRevision and its sub-types
        - Mdl0BaselineRevision and its sub-types
        - Cpd0ShapeDesignRevision and its sub-types
        - Mdl0ApplicationModel and its sub-types
        - Mdl0ModelElement and its sub-types
        - Mdl0AttributeGroup and its sub-types
        - Mdl0ConditionalElement and its sub-types
        - Asp0Aspect and its sub-types
        - Mdl0ManagedAttrGroup and its sub-types
        - Cpd0DesignControlElement and its sub-types
        - Cpd0DesignFeature and its sub-types
        - Cpd0DesignModelElement and its sub-types
        - Ptn0Partition and its sub-types
        
        
        
        Use cases:
        Use Case 1: SaveAs without relate.
        - Client constructs the "SaveAs" dialog for a business object using SaveAs operation descriptor. The
        information returned by that operation allows client to construct the SaveAs dialogs and DeepCopy panels for
        user input.
        - Once the user input is received, client makes subsequent invocation to this operation to execute SaveAs on
        the object. 
        - The method is invoked with "relate" option as false. New object is created using values passed in. It is not
        found under Home / NewStuff folder / anyother parent object. The new object stays dangling.
        Use Case 2: SaveAs and relate to default folder.
        - Client invokes SaveAs operation as mentioned in use case 1 with "relate" as true but chooses not to specify
        target object or relation. 
        - This operation will choose a default folder and choose a default relation to be used. The default folder is
        decided based on the value set for the preference, WsoInsertNoSelectionsPref. 
        - When the preference value is set as 1, the default folder will be the New Stuff folder of the service user. 
        - When the preference value is 2, the default folder will be the Home folder of the service user.
        - Newly created object is related to the default folder using default relation. For any other value of the
        preference, the relation will not be created.
        Use Case 3: SaveAs and relate to specified target object using specified relation.
        - Client invokes SaveAs operation as mentioned in use case 1. The input parameter carrying the relation info
        has boolean flag "relate" which is true, a valid target object and a property name to which the relation is to
        be created.
        - After a successful creation, this operation relates the newly created object to the specified target object
        using specified relation.
        """
        return cls.execute_soa_method(
            method_name='saveAsObjectsInBulkAndRelate',
            library='Internal-Core',
            service_date='2016_10',
            service_name='DataManagement',
            params={'saveAsData': saveAsData, 'relationInfo': relationInfo},
            response_cls=SaveAsObjectsResponse,
        )



from tcsoa.gen.DocumentManagement._2018_11.services import PrintOrRenderService as imp0
from tcsoa.gen.DocumentManagement._2010_04.services import DigitalSignatureService as imp1
from tcsoa.gen.DocumentManagement._2008_06.services import DocumentControlService as imp2
from tcsoa.gen.DocumentManagement._2010_04.services import LaunchDefinitionService as imp3
from tcsoa.gen.DocumentManagement._2013_12.services import PrintOrRenderService as imp4
from tcsoa.gen.DocumentManagement._2007_01.services import DocumentTemplateService as imp5
from tcsoa.gen.DocumentManagement._2011_06.services import AttributeExchangeService as imp6
from tcsoa.gen.DocumentManagement._2018_06.services import AttributeExchangeService as imp7
from tcsoa.utils import TcService


class PrintOrRenderService(TcService):
    containsRenderableFiles = imp0.containsRenderableFiles
    getPrinterDefinitions = imp4.getPrinterDefinitions
    printSubmitRequest = imp4.printSubmitRequest
    renderFilesSubmitRequest = imp0.renderFilesSubmitRequest
    renderSubmitRequest = imp4.renderSubmitRequest


class DigitalSignatureService(TcService):
    digitalSigningSave = imp1.digitalSigningSave


class DocumentControlService(TcService):
    getAdditionalFilesForCheckin = imp2.getAdditionalFilesForCheckin
    getCheckinModeAndFiles = imp2.getCheckinModeAndFiles
    postCreate = imp2.postCreate


class LaunchDefinitionService(TcService):
    getLaunchDefinition = imp3.getLaunchDefinition


class DocumentTemplateService(TcService):
    getTemplates = imp5.getTemplates


class AttributeExchangeService(TcService):
    resolveAttrMappingsAndGetProperties = imp6.resolveAttrMappingsAndGetProperties
    resolveAttrMappingsAndSetProperties = imp6.resolveAttrMappingsAndSetProperties
    updateDocumentProperties = imp7.updateDocumentProperties
